#ifndef RACE_DATA_BLOCK_H
#define RACE_DATA_BLOCK_H
#include<errorhandling/errorhandling.h>
#include<RAMS/mseg.h>
#include<RACE_Exceptions.h>

class RACE_Data_Block;

//##Documentation
//## a real environment friendly FactoryInterface for datablocks
//## Note: A concrete RACE_Data_Block_Factory could optimize the memory allocation
//##       An derived it can return datablocks from a FileReader
class RACE_Data_Block_Factory
{
public:
  virtual RACE_Data_Block *generate(size_t) throw (NoMoreDynamicMemory)=0;
  virtual void recycle(RACE_Data_Block *schrott)=0;
  //##Documentation
  //## a concrete Factory may want to count - default does nothing
  virtual void informduplicate(RACE_Data_Block *);
  //##Documentation
  //## a concrete Factory may want to count - default does nothing
  virtual void informrelease(RACE_Data_Block *);
protected:
  //##Documentation
  //## Only this method in this calls the Constructor of a datablock
  //## withcontent: if buffer contains data (in the case of Readers));
  RACE_Data_Block *create(char *buffer,size_t size,
                    bool withcontent=false);
  //##Documentation
  //## Only this method in this calls the Destructor of a datablock
  void destroy(RACE_Data_Block *dbf);
};

//##Documentation
//##  RACE_Data_Block is a simplified ACE_Data_block
//##  and a RAMS::Mseg to have relative pointers on it.
class RACE_Data_Block : public RAMS::Mseg
{
public:
  size_t length() { return cur_size_; }
  static RACE_Data_Block *alloc(size_t) throw (NoMoreDynamicMemory);
  RACE_Data_Block *clone() throw (NoMoreDynamicMemory);
  RACE_Data_Block *duplicate();
  void release();

  size_t freespace() { return size()-cur_size_; }

  void grow(size_t n) throw (BufferOverflow);

protected:
  //##Documentation
  //## Constructor.
  //## withcontent: if buffer contains data (in the case of Readers)
  RACE_Data_Block(RACE_Data_Block_Factory* dbf,char *buffer,size_t size,bool withcontent=false):
    RAMS::Mseg(buffer,size),
    dbf_(dbf),cur_size_(withcontent ? size :0),
    reference_count_(1),next(0)
  {
  }
  //##Documentation
  //## Destructor
  ~RACE_Data_Block();

  RACE_Data_Block_Factory* dbf_;

  //##Documentation
  //## Current size of message block.
  size_t cur_size_;

  //##Documentation
  //##  Reference count for this <RACE_Data_Block>, which is used to avoid
  //##  deep copies (i.e., <clone>).  Note that this pointer value is
  //##  shared by all owners of the <RACE_Data_Block>'s data, i.e., all the
  //##  <RACE_Message_Block>s.
  //##  I want to use the ACE_Atomic_op, but it don't seems to work ???
  ACE_Thread_Mutex refcountmutex_;
  int reference_count_;

public:
  RACE_Data_Block *next;   // chaining
  friend class RACE_Data_Block_Factory;
};

class RACE_Simple_Malloc_Data_Block_Factory : public RACE_Data_Block_Factory
{
public:
  virtual RACE_Data_Block *generate(size_t) throw (NoMoreDynamicMemory);
  virtual void recycle(RACE_Data_Block *schrott);
};
#endif

