#ifndef DUMPABLE_H_INCLUDED_C1D2A5B5
#define DUMPABLE_H_INCLUDED_C1D2A5B5

#include"RACE_LogfileOffset.h"
#include"RACE_Message_Block.h"
#include"RACE_Data_Block.h"

//##Documentation
//## This is the Interface for classes dumped into a Logfile
class RACE_Dumpable: public RACE_Message_Block
{
  public:
    RACE_Dumpable(size_t size):RACE_Message_Block(size) {}
    RACE_Dumpable(RACE_Data_Block *db,bool frag,RACE_LogfileOffset lo):RACE_Message_Block(db,frag),offset_(lo) {}

    //##Documentation
    //## specifies if the Data is not so important,
    //## thus it can be discarded if too busy
    virtual bool IsDiscardable();

    //##Documentation
    //## When the offset in the Logfile is known,
    //## the Caller must be informed with the offset.
    virtual void InformOffset(RACE_LogfileOffset offset);

    //##Documentation
    //## When the dumpable is written to the Logfile,
    //## the Caller must be informed.
    virtual void InformCompleted();

    //##Documentation
    //## get the Offset - mainly intended for the read side
    RACE_LogfileOffset GetOffset() { return offset_; }

    //##Documentation
    //## If a Dumpable is discarded, potentaly some work has to be done,
    //## for instance
    //##  * a snapshot waits for it,
    //##  * a statistic, on which a strategy can calculate
    //## if the next Dumpable should be despensable
    //## (for instance, for high frequent Clones, each third should
    //## be logged minimaly)
    virtual void Discard();

 

  protected:
    RACE_Dumpable():RACE_Message_Block() {}
    virtual ~RACE_Dumpable();
    virtual RACE_Message_Block *CloneManagementData() const;

    RACE_LogfileOffset offset_;
};

#endif /* DUMPABLE_H_INCLUDED_C1D2A5B5 */
