#include "RACE_LogFileName.h"
#include "errorhandling/errorhandling.h"
#include <cstdio>

RACE_LogFileName::RACE_LogFileName (const char* basename, const char* extension) {
  set (basename, extension);
}

RACE_LogFileName::RACE_LogFileName (const char* basenamepath, const char* basenamefile, const char* extension) {
  RACE_COND_FAIL ((basenamepath == 0) || (basenamefile == 0),
                     "RACE_LogFileName::RACE_LogFileName: parameter missing");
  unsigned buffersize = strlen (basenamepath) + strlen (basenamefile) + 2;
  char buffer [buffersize];
  sprintf (buffer, "%s/%s", basenamepath, basenamefile);
  set (buffer, extension);
}

RACE_LogFileName::RACE_LogFileName (const RACE_LogFileName& original) {
  m_basename = NULL;
  m_extension = NULL;
  m_fullname = NULL;
  (*this) = original;
}

RACE_LogFileName&
RACE_LogFileName::operator= (const RACE_LogFileName& original) {
  reset ();
  set (original.m_basename, original.m_extension);
  return *this;
}

RACE_LogFileName::~RACE_LogFileName () {
  reset ();
}

char* RACE_LogFileName::get_versioned_fullname (unsigned version) const {
  static const unsigned max_unsigned_len = 10;
  char* versioned_fullname = new char [strlen (m_basename) + max_unsigned_len + strlen (m_extension) + 3];
  RACE_COND_FAIL (versioned_fullname == NULL, "%s:%d: memory allocation failure", __FILE__, __LINE__);
  sprintf (versioned_fullname, "%s_%u.%s", m_basename, version, m_extension);
  return versioned_fullname;
}

char* RACE_LogFileName::get_copiedfullname () const {
  char* copiedfullname = new char [strlen (m_fullname) + 1];
  RACE_COND_FAIL (copiedfullname == NULL, "%s:%d: memory allocation failure", __FILE__, __LINE__);
  sprintf (copiedfullname, "%s", m_fullname);
  return copiedfullname;
}

void
RACE_LogFileName::set (const char* basename, const char* extension) {
  RACE_COND_FAIL (basename == NULL, "RACE_LogFileName::set: missing basename parameter");
  RACE_COND_FAIL (extension == NULL, "RACE_LogFileName::set: missing extension parameter");
  m_basename = strdup (basename);
  RACE_COND_FAIL (m_basename == NULL, "RACE_LogFileName::set: could not assign member m_basename");
  m_extension = strdup (extension);
  RACE_COND_FAIL (m_extension == NULL, "RACE_LogFileName::set: could not assign member m_extension");
  m_fullname = new char [strlen (m_basename) + strlen (m_extension) + 2];
  RACE_COND_FAIL (m_fullname == NULL, "%s:%d: memory allocation failure", __FILE__, __LINE__);
  sprintf (m_fullname, "%s.%s", m_basename, m_extension);
}

void 
RACE_LogFileName::reset () {
  if (m_basename) {
    free (m_basename);
    m_basename = NULL;
  }
  if (m_extension) {
    free (m_extension);
    m_extension = NULL;
  }
  if (m_fullname) {
    delete [] m_fullname;
    m_fullname = NULL;
  }
}

