#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <stdio.h>
#include <errno.h>

#include "RACE_LogIndexPositioner.h"
#include "errorhandling/envdebug.h"
#include "errorhandling/errorhandling.h"

static envdebug *s_debug=new envdebug("RACE_LogIndexPositioner");

RACE_LogIndexPositioner::RACE_LogIndexPositioner(const char *filename)
{
  struct stat sb;
  m_fd=::open(filename,O_RDONLY);
  if(m_fd>=0) {
    int r=fstat(m_fd,&sb);
    RACE_COND_FAIL(r<0,"%s:%d: Cann't fstat %s : %s",__FILE__,__LINE__,
                   filename,strerror(errno));
    mp_entries =(RACE_LogIndexLayout *)mmap(NULL,sb.st_size,PROT_READ,MAP_SHARED,m_fd,0);
    RACE_COND_FAIL(mp_entries<0,"%s:%d: Cann't fstat %s : %s",__FILE__,__LINE__,
                   filename,strerror(errno));
    m_count = sb.st_size/sizeof(RACE_LogIndexLayout);
  }
  else {
    RACE_ERROR("%s:%d: Cann't Open %s : %s",__FILE__,__LINE__, filename,strerror(errno));
    mp_entries = 0;
    m_count = 0;
  }
}

const char *RACE_LogIndexPositioner::RangeError::What()
{
  snprintf(m_errstr, sizeof(m_errstr),
           "RACE_LogIndexPositioner: RangeError %u (of %u)", m_index, m_max);
  return m_errstr;
}

bool RACE_LogIndexPositioner::Open(const char *filename)
{
  struct stat sb;
  int r;
  m_fd = ::open(filename,O_RDONLY);
  if (m_fd < 0)
  {
    RACE_ERROR("RACE_LogIndexPositioner::Open: %s:%d: can't open %s : %s", __FILE__, __LINE__,
            filename, strerror(errno));
    return false;
  }
  r = fstat(m_fd,&sb);
  if (r < 0)
  {
    RACE_ERROR("RACE_LogIndexPositioner::Open: %s:%d: can't fstat %s : %s", __FILE__, __LINE__,
            filename, strerror(errno));
    return false;
  }
  mp_entries =(RACE_LogIndexLayout *)mmap(NULL,sb.st_size,PROT_READ,MAP_SHARED,m_fd,0);
  if (mp_entries < 0)
  {
    RACE_ERROR("RACE_LogIndexPositioner::Open: %s:%d: Cann't fstat %s : %s", __FILE__, __LINE__,
            filename, strerror(errno));
    return false;
  }
  m_count = sb.st_size / sizeof(RACE_LogIndexLayout);
  return true;
}

void RACE_LogIndexPositioner::Close()
{
  if (m_fd > 0)
  {
    close(m_fd);
    m_fd = -1;
  }
}

RACE_LogIndexPositioner::~RACE_LogIndexPositioner()
{
  munmap((void *)mp_entries,m_count*sizeof(RACE_LogIndexLayout));
  Close();
}

void RACE_LogIndexPositioner::Print()
{
  for(unsigned i=0;i<m_count;i++)
  {
    struct tm *ts = localtime((time_t *)&(mp_entries[i].unixtime));
    printf("%u. %d:%.2d:%.2d (%d) %d:%d\n",i,
                  ts->tm_hour,ts->tm_min,ts->tm_sec,
                  mp_entries[i].unixtime,mp_entries[i].offset.part,
                  mp_entries[i].offset.offset);
  }
}

RACE_LogIndexLayout& RACE_LogIndexPositioner::operator[](unsigned i) throw (RACE_LogIndexPositioner::RangeError)
{
  if(i<0 || i>=m_count) throw RACE_LogIndexPositioner::RangeError(i,m_count);
  return mp_entries[i];
}

unsigned RACE_LogIndexPositioner::GetLastIndex(time_t wst)
{
  unsigned li=0;
  if(m_count<2 || wst < mp_entries[0].unixtime) return 0;
  if(wst >= mp_entries[m_count-1].unixtime) return m_count-1;
  int T = (mp_entries[m_count-1].unixtime - mp_entries[0].unixtime)/m_count;
  li=(wst-mp_entries[0].unixtime)/T;

  while(li<m_count && mp_entries[li].unixtime < wst) li++;
  while(li>0 && mp_entries[li].unixtime > wst) li--;
  return li;
}

time_t RACE_LogIndexPositioner::StrToTime(const char *hhmmss)
{
  time_t ret;
  int r;
  struct tm tmib;
  s_debug->print("StrToTime: First time: %d ",mp_entries[0].unixtime);
  const int32_t *unixtime = &(mp_entries[0].unixtime);
  struct tm *tmi=localtime_r((const time_t*)unixtime, &tmib);
  tmi->tm_hour=0;
  tmi->tm_min=0;
  tmi->tm_sec=0;
  r=sscanf(hhmmss,"%d:%d:%d",&(tmi->tm_hour),&(tmi->tm_min),&(tmi->tm_sec));
  s_debug->print("RACE_LogIndexPositioner::StrToTime: scanned: %d - %d.%d.%d %d:%d:%d",
                 r,tmi->tm_mday,tmi->tm_mon+1,tmi->tm_year+1900,tmi->tm_hour,tmi->tm_min,tmi->tm_sec);
  ret=mktime(tmi);
  s_debug->print("StrToTime: time=%ld",ret);
  return ret;
}
