#ifndef RACELOGINDEXPOSITIONER_H
#define RACELOGINDEXPOSITIONER_H
#include "RACE_LogIndexLayout.h"

//##Documentation
//##  Reads the indexfile and maps the starting time to offset of the nearest
//##  older Snapshot
class RACE_LogIndexPositioner
{
public:
  class RangeError
  {
  public:
    RangeError(unsigned index,unsigned max)
      : m_index(index), m_max(max) {}
    unsigned GetIndex() const { return m_index; }
    unsigned GetMax() const { return m_max; }
    const char *What();
  private:
    unsigned m_index;
    unsigned m_max;
    char m_errstr[80];
  };
  RACE_LogIndexPositioner() : m_fd(-1), mp_entries(0), m_count(0) {}
  RACE_LogIndexPositioner(const char *filename);

  //##Documentation
  //## Opens the index file.  Returns true for success.
  bool Open(const char *filename);

  //##Documentation
  //## Closes the index file.
  void Close();

  ~RACE_LogIndexPositioner();

  //##Documentation
  //##  Prints the Index on stdout
  void Print();

  //##Documentation
  //##  gets the number of Entries in the Indexfile
  unsigned GetCount() { return m_count; }

  //##Documentation
  //##  gets the i-th entry
  RACE_LogIndexLayout& operator[](unsigned i) throw (RACE_LogIndexPositioner::RangeError);

  //##Documentation
  //## Gets the last Index before time wst
  unsigned GetLastIndex(time_t wst);

  //##Documentation
  //##  Converts a string of the Form hh:mm:ss into a Timestamp
  //##  The Date is fetched from the first entry and the time is overwritten
  //##  with the parsed time from the string
  time_t StrToTime(const char *hhmmss);

  //##Documentation
  //##  gets offset from corresponding Timestamp
  RACE_LogfileOffset Get(time_t wst) { return (*this)[GetLastIndex(wst)].offset; }

  //##Documentation
  //## Get the last offset in the Index
  RACE_LogfileOffset GetLastOffset() { return (*this)[m_count-1].offset; }

  //##Documentation
  //## checks if the index is opened and Entries in the Index
  bool IsEmpty() { return m_fd<0 || m_count<=0; }
protected:
  int m_fd;
  RACE_LogIndexLayout *mp_entries;
  unsigned m_count;
};
#endif
