#ifndef RACE_LOGSWITCHINGSTRATEGY_H
#define RACE_LOGSWITCHINGSTRATEGY_H
#include <sys/types.h>
#include <errorhandling/envdebug.h>
#include "RACE_LogSwitcher.h"

//##Documentation
//##  There are serveral strategies thinkable, when to begin a new part
//##  of the logfile:
//##  - only when needed at a hard_limit
//##  - when a softlimit is reached and a checkpoint is written
//##  - periodic, for example every hour
//##  This defines the interface of the strategy.
//##  Thus, a special Strategy may be specified even in a logging_script

class RACE_LogSwitchingStrategy
{
public:
  virtual ~RACE_LogSwitchingStrategy () {}
  RACE_LogSwitchingStrategy(RACE_LogSwitcher *ls):mp_LogSwitcher(ls),m_Hardlimit(1<<30) {}
  void SetSwitcher(RACE_LogSwitcher *ls) { mp_LogSwitcher=ls; }
  virtual void InfoCurrentOffset(off_t offset);
  virtual void InfoSnapshotWritten()=0;
protected:
  //##Documentation
  //## All concrete derivations log over this Variable
  static envdebug *ms_debug;
  RACE_LogSwitcher *mp_LogSwitcher;
  off_t m_Hardlimit;
  off_t m_CurrentOffset;
};

class RACE_DefaultLogSwitchingStrategy : public RACE_LogSwitchingStrategy
{
  off_t m_Softlimit;
public:
  RACE_DefaultLogSwitchingStrategy(RACE_LogSwitcher *ls);
  virtual void InfoSnapshotWritten();
};
#endif

