#include "RACE_Logfile.h"
#include<unistd.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<string.h>
#include <errorhandling/errorhandling.h>
#include<error.h>


RACE_Logfile::RACE_Logfile() : m_fd(-1), m_partno(0)
{
  m_filename[0]='\0';
  m_bodyname[0]='\0';
}

RACE_Logfile::~RACE_Logfile()
{
  if(m_fd!=-1) Close();
  index_.Close();
}

void RACE_Logfile::SetFilenameBody(const char *bodyname)
{
  strncpy(m_bodyname,bodyname,128);
  m_bodyname[127]='\0';
}

const char *RACE_Logfile::GetBasename() const
{
  return m_filename;
}

void RACE_Logfile::Open() throw (Systemerror)
{
  RACE_COND_FAIL(m_bodyname[0]=='\0',"RACE_Logfile::Open(): no bodyname set");
  if(m_fd==-1)
  {
    time_t t;
    time(&t);
    struct tm *tmi=localtime(&t);
    snprintf(m_filename,128,"%s-%d%.2d%.2d-%.2d:%.2d:%.2d",m_bodyname,
             (tmi->tm_year+1900),(tmi->tm_mon+1),tmi->tm_mday,
             tmi->tm_hour,tmi->tm_min,tmi->tm_sec
             );
    m_partno=0;
    OpenPart();
    index_.Open(GetBasename());
  }
  else
  {
    // Already open
  }
}

void RACE_Logfile::Open(const char *filename) throw (Systemerror)
{
  m_fd=-1; // force Open to realy open a new file
  SetFilenameBody(filename);
  Open();
}


void RACE_Logfile::OpenPart() throw (Systemerror)
{
  int filenamelen=strlen(m_filename)+20;
  char *filename = (char *)alloca(filenamelen);
  RACE_COND_FAIL(filename==NULL,"%s:%d: alloca(%d) failed",
                    __FILE__,__LINE__,filenamelen);
  m_partno++;
  snprintf(filename,filenamelen,"%s_%d.log",m_filename,m_partno);
  if(m_fd>0) { ::close(m_fd); }
  m_fd=::open(filename,O_CREAT|O_WRONLY,0644);
  if(m_fd < 0)
  {
      throw Systemerror(errno,"open","RACE_Logfile::open: can't open %s:%m",filename);
  }
}

void RACE_Logfile::Close()
{
  ::close(m_fd);m_fd=-1;
}

int RACE_Logfile::WriteV(const iovec *io,int count) throw (Systemerror)
{
  int ret=::writev(m_fd,io,count);
  if(ret<0) throw Systemerror(errno,"writev","RACE_Logfile::WriteV(%s_%d: %m",
			      GetBasename(),PartNo());
  return ret;
}

int RACE_Logfile::PartNo()
{
   return m_partno;
}

void RACE_Logfile::SwitchPart() throw (Systemerror)
{
  OpenPart();
}


