#include<errorhandling/errorhandling.h>
#include"RACE_Message_Block.h"
#include<iostream>
#include<string.h>
#include<stdlib.h>

RACE_Message_Block::RACE_Message_Block (size_t len):block_size_ (len)
{
  first_ = db = RACE_Data_Block::alloc (len);
  reset ();
  _refcount = 1;
  next_ = 0;
  prev_ = 0;
}

RACE_Message_Block::RACE_Message_Block(RACE_Data_Block *edb,bool /*frag*/)
  :db(edb),first_(edb),
   block_size_(edb->size()),
   rd_ptr_(0),
   wr_ptr_(edb->length()),
   _refcount(1),
   next_(0),prev_(0)
{
}

RACE_Message_Block::~RACE_Message_Block ()
{
  RACE_Data_Block *temp = NULL;
  RACE_Data_Block *cur = first_;
  while (cur != NULL)
  {
    temp = cur;
    cur = cur->next;
    temp->release ();
  }
}

void
RACE_Message_Block::reset ()
{
  rd_ptr_ = 0;
  wr_ptr_ = 0;
}

char*
RACE_Message_Block::base () const
{
  return (char*)db->begin ();
}

char*
RACE_Message_Block::end () const
{
  return ((char*)db->begin ())+db->length ();
}

char*
RACE_Message_Block::first_base () const
{
  return (char*)first_->begin ();
}

int
RACE_Message_Block::copy (const char *buf, size_t n) throw (BufferOverflow)
{
  memcpy(wr_ptr(n),buf,n);
  return n;
}

char*
RACE_Message_Block::rd_ptr () const
{
  return base ()+rd_ptr_;
}

void
RACE_Message_Block::rd_ptr (char *ptr)
{
  rd_ptr_ = ptr-base ();
}

char *
RACE_Message_Block::rd_ptr (size_t n)
{
  char *ret=base()+rd_ptr_;
  rd_ptr_ += n;
  return ret;
}

char*
RACE_Message_Block::wr_ptr () const
{
  return base ()+wr_ptr_;
}

char*
RACE_Message_Block::wr_ptr (char *ptr) throw (BufferOverflow)
{
  size_t neu = ptr-base ();
  if (neu>db->length ()) db->grow (neu - db->length ());
  wr_ptr_ = neu;
  return wr_ptr ();
}

char*
RACE_Message_Block::wr_ptr (size_t n) throw (BufferOverflow)
{
  char *ret=wr_ptr();
  wr_ptr_ += n;
  if (wr_ptr_>db->length ()) db->grow (wr_ptr_ - db->length ());
  return ret;
}

char *RACE_Message_Block::wrptr_align(short potenz2)
{
  char *old=wr_ptr();
  long a=1<<potenz2;
  long mask=a-1;
  try
  {
    RACE_COND_FAIL(sizeof(char *)!=sizeof(long),"%s:%d:sizeof(char *)!=sizeof(long)",__FILE__,__LINE__);
    wr_ptr((char *)((long(old)+mask)&(~mask)));
    memset(old,'\0',wr_ptr()-old);
  }
  catch(BufferOverflow& ex)
  {
    wr_ptr(old);
  }
  return wr_ptr();
}

size_t
RACE_Message_Block::size () const
{
  return db->size ();
}

size_t
RACE_Message_Block::length () const
{
  return db->length ();
}

size_t
RACE_Message_Block::remaining () const
{
  return wr_ptr_-rd_ptr_;
}

size_t
RACE_Message_Block::total_length () const
{
  size_t ret = 0;
  for (RACE_Data_Block *cur = first_;cur != NULL;cur = cur->next)
  {
    ret += cur->length ();
  }
  return ret;
}

size_t
RACE_Message_Block::total_size () const
{
  size_t ret = 0;
  for (RACE_Data_Block *cur = first_;cur != NULL;cur = cur->next)
  {
    ret += cur->size ();
  }
  return ret;
}

void
RACE_Message_Block::expand ()
{
  db->next = RACE_Data_Block::alloc (block_size_);
  db = db->next;
  reset ();
}

void RACE_Message_Block::addblock(RACE_Data_Block *block)
{
  db->next=block;
  db=db->next;
  reset();
}

bool
RACE_Message_Block::IsFragmented () const
{
  return (first_ != NULL && first_->next != NULL);
}

void
RACE_Message_Block::begin ()
{
  db = first_;
  if (db) reset_current ();
}

bool
RACE_Message_Block::is_done () const
{
  return db == NULL;
}

bool
RACE_Message_Block::is_last () const
{
  if (db) return (db->next == NULL);
  return false;
}

void
RACE_Message_Block::advance ()
{
  db = db->next;
  if (db) reset_current ();
}

RACE_Message_Block*
RACE_Message_Block::add_ref ()
{
  ACE_GUARD_RETURN (ACE_Thread_Mutex, guard, this->_refcountmutex, 0);
  _refcount++;
  return this;
}

RACE_Message_Block*
RACE_Message_Block::duplicate () const
{
  RACE_Message_Block *ret = CloneManagementData ();
  ret->first_ = first_;
  ret->db = db;
  ret->reset_current ();
  for (RACE_Data_Block *cur = first_;cur != NULL;cur = cur->next)
  {
    cur->duplicate ();
  }
  return ret;
}

RACE_Data_Block *RACE_Message_Block::duplicate_chain()
{
  RACE_Data_Block *ret = first_;
  for (RACE_Data_Block *cur = first_;cur != NULL;cur = cur->next)
  {
    cur->duplicate ();
  }
  return ret;
}

RACE_Data_Block *
RACE_Message_Block::firstblock()
{
    return first_;
}
RACE_Data_Block *
RACE_Message_Block::curblock()
{
    return db;
}

RACE_Message_Block*
RACE_Message_Block::clone () const
{
  RACE_Message_Block* ret = CloneManagementData ();
  ret->block_size_ = block_size_;
  ret->first_ = NULL;
  RACE_Data_Block **cur_insert = &(ret->first_);
  for (RACE_Data_Block *cur = first_;cur != NULL;cur = cur->next)
  {
    ret->db = *cur_insert = cur->clone ();
    ret->reset_current ();
    cur_insert = &((*cur_insert)->next);
  }
  return ret;
}

void
RACE_Message_Block::release ()
{
  unsigned current_ref_count = 0;
  {
    ACE_GUARD(ACE_Thread_Mutex, guard, this->_refcountmutex)
    current_ref_count = --_refcount;
  }
  if (current_ref_count == 0) {
    delete this;
  }
}

RACE_Message_Block*
RACE_Message_Block::next () const
{
  return next_;
}

void
RACE_Message_Block::next (RACE_Message_Block *n)
{
  next_ = n;
}

RACE_Message_Block*
RACE_Message_Block::prev () const
{
  return prev_;
}

void
RACE_Message_Block::prev (RACE_Message_Block *p)
{
  prev_ = p;
}

RACE_Message_Block*
RACE_Message_Block::CloneManagementData () const
{
  RACE_Message_Block *ret = new RACE_Message_Block ();
  RACE_COND_FAIL (ret == NULL,"%s%d:RACE_Message_Block::CloneManagementData (): No More Memory", __FILE__,__LINE__);
  return ret;
}

RACE_Message_Block::RACE_Message_Block ()
{
  first_ = db = (RACE_Data_Block *)0;
  reset ();
  _refcount = 1;
  next_ = 0;
  prev_ = 0;
}

void
RACE_Message_Block::reset_current ()
{
  rd_ptr_ = 0;
  wr_ptr_ = db->length ();
}

void RACE_Message_Block::CallVisitor(RACE_MessageVisitor * /*visitor*/)
{
}

#ifdef TEST_DATA_BLOCK
int main ()
{
  std::cerr << "sizeof (RACE_Data_Block)=" << sizeof (RACE_Data_Block) << std::endl;
  RACE_Data_Block* rdb = RACE_Data_Block::alloc (4096);
  std::cerr << "maxsize=" << rdb->max_size () << std::endl;
}
#endif

