#include<ace/OS_NS_time.h>
#include"RACE_PeriodicWorkerThread.h"
#include<errorhandling/envdebug.h>

using namespace RACE;

PeriodicWorkerThread::PeriodicWorkerThread(RACE::Periodic_Work *pw):m_stop(true),pw_(pw)
{
}

void PeriodicWorkerThread::start()
{
    // Note: it is called here, since we don't know in which order the
    // Threads ar starting. This is important when an other thread calls
    // a Method in an derivation of RACE_Periodic_Work
    pw_->atstart();
    {
	ACE_GUARD(ACE_Thread_Mutex,guard,mutex_);
	m_stop=false;
    }
    RACE_Thread::start();
}

void PeriodicWorkerThread::stop()
{
    debug->print("PeriodicWorkerThread::stop()");
    ACE_GUARD(ACE_Thread_Mutex,guard,mutex_);
    m_stop=true;
}

bool PeriodicWorkerThread::is_stopped()
{
    ACE_GUARD_RETURN(ACE_Thread_Mutex,guard,mutex_,true);
    return m_stop;
}

void PeriodicWorkerThread::run()
{
    int ret;
    while(!is_stopped())
    {
		ACE_Time_Value duration,endsleeptime,currenttime;
	ACE_Time_Value starttime = ACE_OS::gettimeofday();
	ret=pw_->do_work(starttime);
	currenttime = ACE_OS::gettimeofday();
	endsleeptime = starttime+pw_->getPeriod();
	duration=starttime-currenttime;
        if(ret==0)
        {
	    /*
             * to prevent long waiting time in case of stop
	     * we are waiting only maximal 1.25 seconds
             */
	    while(!is_stopped() && currenttime < endsleeptime)
	    {
		struct timespec towait = endsleeptime-currenttime;
		if(towait.tv_nsec > 250000)
		{
		    towait.tv_sec=0;
		}
		if(towait.tv_sec>1) 
		{
		    towait.tv_sec=1;
		} 
		ACE_OS::nanosleep(&towait);
		currenttime=ACE_OS::gettimeofday();
	    }
        }
        else
        {
           debug->print("do_work() returned -1 ");
           stop();
        }
    }
    debug->print("RACE::PeriodicWorkerThread stopped");
}
