#ifndef RACE_PERIODIC_WORK
#define RACE_PERIODIC_WORK
#include<ace/Time_Value.h>
#include<ace/Event_Handler.h> // for EventHandlerAdapter
#include<ace/Reactor.h>

namespace RACE
{
    //##Documentation
    //## interface for periodic work 
    //## Can be called in a Thread or in the ACE_Reactor
    class Periodic_Work
    {
    public:
	Periodic_Work(ACE_Time_Value periode);
	virtual ~Periodic_Work();
        //##Documantation
	//## call at starting
        //## A Periodic Work can be started more than once,
        //## for instance, an alivechecker is only active when a observer
	//## registered
        virtual void atstart();
	//##Documentation
	//## returnvalue like in Methods ACE_Event_Handler:
	//## 0: ok, remain in working
        //## <0: leave fast , >0 leave slowly
	virtual int do_work(const ACE_Time_Value& current_time)=0;
	//##Documentation
	//## get the periode, in that do_work shall be called.
	//## default-Implementation: constant with value given by constructor
	virtual ACE_Time_Value getPeriod();
	//Concept Priority ???
    private:
	ACE_Time_Value period_;
    };

    class EventHandlerAdapter : public ACE_Event_Handler
    {
    public:
       EventHandlerAdapter(Periodic_Work *work);
       ~EventHandlerAdapter();
       virtual int handle_timeout(const ACE_Time_Value &current_time,
                              const void *act = 0);
       long register_at_reactor(ACE_Reactor *reactor);
       void cancel_timer();
    private:
       Periodic_Work *work_;       
       long timerid_;
       ACE_Reactor *reactor_;
    };
} /* namespace RACE */
#endif
