#include "RACE_QueueLevelStrat.h"
#include "errorhandling/envdebug.h"
#include "errorhandling/errorhandling.h"

static envdebug *debug = new envdebug("RACE_QueueLevelStrategy");

void RACE_QueueLevelStrategy::add(size_t cur_size,size_t cur_length)
{
  cur_count_++;
  cur_lengthsum_ += cur_length;
  cur_sizesum_   += cur_size;
  debug->print("RACE_QueueLevelStrategy::add after count=%d length=%d bytes size=%d bytes",cur_count_,cur_lengthsum_,cur_sizesum_);
  if(is_critical_watermark_reached())
  {
    if(!critical_watermark_triggered)
    {
      critical_watermark_alarm();
      critical_watermark_triggered=true;
    }
  }
  else
  {
    // Alarm is over
    critical_watermark_triggered=false;
  }
}

void RACE_QueueLevelStrategy::remove(size_t cur_size,size_t cur_length)
{
  RACE_COND_FAIL(cur_count_<1,
                    "RACE_QueueLevelStrategy::remove: Queuesize<0");
  cur_count_--;  // assert cur_count_ >=0
  cur_lengthsum_ -= cur_length;
  cur_sizesum_   -= cur_size;
}

bool RACE_QueueLevelStrategy::is_critical_watermark_reached()
{
  return false;
}

const char *RACE_QueueLevelStrategy::GetQueueName()
{
  return "";
}

void RACE_QueueLevelStrategy::critical_watermark_alarm()
{
  RACE_WARN("Critical Watermark in Queue %s is reached",GetQueueName());
}

ACE_Time_Value *RACE_QueueLevelStrategy::GetCurrentEnqueueTimeout()
{
  return 0;
}

ACE_Time_Value *RACE_QueueLevelStrategy::GetCurrentDequeueTimeout()
{
  return 0;
}

RACE_QueueLevelStrategy::~RACE_QueueLevelStrategy() {}

bool RACE_QLS_Count::can_enqueue()
{
  return cur_count_ < max_count_;
}

bool RACE_QLS_Count::is_full()
{
  return cur_count_ >= max_count_;
  // >: defensive programming
}

bool RACE_QLS_MessageSizeSum::can_enqueue()
{
  return cur_sizesum_ <= this->low_watermark_;
}

bool RACE_QLS_MessageSizeSum::is_full()
{
  return cur_sizesum_ > high_watermark_;
}


RACE_QLS_Count::~RACE_QLS_Count()
{
}

RACE_QLS_MessageSizeSum::~RACE_QLS_MessageSizeSum()
{
}
