#ifndef RACE_QUEUELEVELSTRAT_H
#define RACE_QUEUELEVELSTRAT_H
#include<sys/types.h>
#include<ace/Time_Value.h>

//##Documentation
//## This is the strategy class which controls the RACE_Message_Queue.
class RACE_QueueLevelStrategy
{
public:
  RACE_QueueLevelStrategy():cur_sizesum_(0),cur_lengthsum_(0), cur_count_(0),
    critical_watermark_triggered(false) {}
  void reset() { cur_sizesum_=0;cur_lengthsum_=0;cur_count_=0; }
  void add(size_t cur_size,size_t cur_length);
  void remove(size_t cur_size,size_t cur_length);
  size_t count()     { return cur_count_; }
  size_t size_sum()  { return cur_sizesum_; }
  size_t length_sum(){ return cur_lengthsum_; }

  bool is_empty() { return cur_count_ == 0; }
  virtual bool can_enqueue()=0;
  virtual bool is_full() = 0;

  virtual bool is_critical_watermark_reached();

  //##Documentation
  //## Get the QueueName for Debugging and Warning purpose
  virtual const char *GetQueueName();

  //##Documentation
  //## A callback function which is called in add()
  //## when is_critical_watermark_reached()
  //## Derived stategies can overwrite this
  virtual void critical_watermark_alarm();


  //##Documentation
  //## Class ACE_Condition requires an absolute time.
  //## The current absolute time is calculated here.
  //## 0 for no timeout
  //## Default is 0
  virtual ACE_Time_Value *GetCurrentEnqueueTimeout();

  //##Documentation
  //## Class ACE_Condition requires an absolute time.
  //## The current absolute time is calculated here.
  //## 0 for no timeout
  //## Default is 0
  virtual ACE_Time_Value *GetCurrentDequeueTimeout();

  virtual ~RACE_QueueLevelStrategy();
protected:
//##Documentation
//## Sum of Message sizes bytes in the queue.
  size_t cur_sizesum_;

//##Documentation
//## Current length of messages in the queue.
  size_t cur_lengthsum_;

//##Documentation
//## Current number of messages in the queue.
  size_t cur_count_;


  bool critical_watermark_triggered;
};

class RACE_QLS_Count :public RACE_QueueLevelStrategy
{
public:
  RACE_QLS_Count(size_t max_count):max_count_(max_count) {}
  virtual bool can_enqueue();
  virtual bool is_full();
  virtual ~RACE_QLS_Count();
protected:
  size_t max_count_;
};

//##Documentation
//##   The <high_water_mark>
//##  determines how many bytes can be stored in a queue before it's
//##  considered "full."  Supplier threads must block until the queue
//##  is no longer full.  The <low_water_mark> determines how many
//##  bytes must be in the queue before supplier threads are allowed to
//##  enqueue additional <RACE_Message_Block>s.  By default, the
//##  <high_water_mark> equals the <low_water_mark>, which means that
//##  suppliers will be able to enqueue new messages as soon as a
//##  consumer removes any message from the queue.  Making the
//##  <low_water_mark> smaller than the <high_water_mark> forces
//##  consumers to drain more messages from the queue before suppliers
//##  can enqueue new messages, which can minimize the "silly window
//##  syndrome."

class RACE_QLS_MessageSizeSum : public RACE_QueueLevelStrategy
{
 public:
  RACE_QLS_MessageSizeSum(size_t low_watermark,size_t high_watermark)
    :low_watermark_(low_watermark),high_watermark_(high_watermark)
    {
    }
  virtual ~RACE_QLS_MessageSizeSum();
  virtual bool can_enqueue();
  virtual bool is_full();
 protected:
  size_t low_watermark_;
  size_t high_watermark_;
};
#endif
