#include "RACE_SnapshotTrigger.h"
#include "errorhandling/envdebug.h"
#include "errorhandling/errorhandling.h"

static envdebug *s_debug = new envdebug("RACE_SnapshotTrigger");

int RACE_SnapshotCreator::GetPendingTotal()
{
  return ST_UNKOWN;
}

int RACE_SnapshotCreator::GetPendingForYou()
{
  return ST_UNKOWN;
}

RACE_SnapshotCreator::~RACE_SnapshotCreator()
{
}

RACE_SnapshotQueue::Entry::~Entry()
{
}

RACE_SnapshotQueue::~RACE_SnapshotQueue()
{
  s_debug->print("RACE_SnapshotQueue::~RACE_SnapshotQueue()");
  Entry* h;
  while(mp_first!=NULL)
  {
    h=mp_first;
    mp_first = mp_first->mp_next;
    delete h;
  }
}


void RACE_SnapshotQueue::GetCountAndSum(int& count,int& sumpending)
{
  count=0;
  sumpending=0;
  for(Entry *a=mp_first;a!=NULL;a=a->mp_next)
  {
    sumpending+=a->GetPending();
    count++;
  }
}

void RACE_SnapshotQueue::enqueueSnapshotEntry(RACE_SnapshotQueue::Entry *e)
{
  *mpp_last = e;
  mpp_last=&((*mpp_last)->mp_next);
}

void RACE_SnapshotQueue::dequeueSnapshotEntry()
{
  if (mp_first) {
    Entry *current_entry = mp_first;
    mp_first=mp_first->mp_next;
    if(mp_first==NULL)
    {
      mpp_last=&mp_first;
    }
    current_entry->mp_next=NULL;
    current_entry->ConnectionLog();
    delete current_entry;
  }
}

RACE_SnapshotQueue::Entry *RACE_SnapshotQueue::GetEntry(time_t timestamp)
{
  Entry *cur=mp_first;
  while(cur!=NULL && cur->GetTimestamp() < timestamp)
  {
    cur=cur->mp_next;
  }
  if(cur!=NULL && cur->GetTimestamp() == timestamp)
  {
    return cur;
  }
  else
  {
    return NULL;
  }
}

/*---------------------- RACE_SnapshotTrigger --------------------*/
RACE_SnapshotTrigger::RACE_SnapshotTrigger
   (const char *description, RACE_SnapshotQueue *sq)
  : m_description(description), m_lastTimestamp(0), m_queue(sq)
{
  if(sq==NULL)
  {
    s_debug->print("RACE_SnapshotTrigger::RACE_SnapshotTrigger(%s): Using Default Queue",description);
    m_queue = new RACE_SnapshotQueue();
    RACE_COND_FAIL(m_queue==NULL,"%s:%d: No Memory ",__FILE__,__LINE__);
  }
}

RACE_SnapshotTrigger::~RACE_SnapshotTrigger()
{
  s_debug->print("RACE_SnapshotTrigger::~RACE_SnapshotTrigger()");
  delete m_queue;
}



void RACE_SnapshotTrigger::PrepareSnapshot(RACE_SnapshotCreator* snapcreator, time_t timestamp)
{
  envdebug::component dbg(s_debug, "RACE_SnapshotTrigger::PrepareSnapshot");
  dbg.print("%s", m_description);
  RACE_COND_FAIL (snapcreator == NULL, "RACE_SnapshotTrigger::PrepareSnapshot: missing snapcreator");

  int inothers=0;
  int count=0;
  int pending;
  int pendingforme;
  ACE_GUARD(ACE_Thread_Mutex, ace_mon, this->m_mutex);

  /*
   * The order of timestamps must be monoton
   * Multiple Perparations of the same snapshots occur in the
   * case of collecting WholeSnapshots
   */
  RACE_COND_FAIL (timestamp < m_lastTimestamp,
                     "RACE_SnapshotTrigger::PrepareSnapshot(%s,timestamp=%ld)<%ld",
                     m_description,timestamp,m_lastTimestamp);

  if(timestamp==m_lastTimestamp)
  {
    dbg.print("(%s,timestamp=%ld) already prepared",m_description,timestamp);
    return;
  }
  m_lastTimestamp = timestamp;

  m_queue->GetCountAndSum(count,inothers);

  if(snapcreator->GetPendingForYou()==ST_UNKOWN)
  {
    /*
     * We know how many Dumpables pending total, but not for the current
     * Snapshot
     * It is in the case of OnEventSnapshots
     */
    pending      = snapcreator->GetPendingTotal();
    pendingforme = pending - inothers;
    dbg.print("(%s %ld)-TN %d Dumpables for %d Snapshots pending, %d for current",
                   m_description,timestamp,pending,count,pendingforme);
  }
  else
  {
    /*
     * We know the number of pending Dumpables for the Current Snapshot
     */
    pendingforme = snapcreator->GetPendingForYou();
    pending = pendingforme+inothers;
    dbg.print("(%s %ld)-CN %d Dumpables for %d Snapshots pending, %d for current",
    m_description,timestamp,pending,count,pendingforme);
  }

  m_queue->enqueueSnapshotEntry(snapcreator->GenNewEntry(timestamp,pendingforme));
  if(pending<=0)  // < defensiv
  {
    dbg.print("(%s): no clones pending - logging", m_description);
    m_queue->dequeueSnapshotEntry();
  }
}

void RACE_SnapshotTrigger::InformInsertConclusion()
{
  envdebug::component dbg(s_debug, "RACE_SnapshotTrigger::InformInsertConclusion");
  dbg.print("%s", m_description);
  int c=1;
  while(!m_queue->IsEmpty() && m_queue->GetTopPending() == 0)
  {
    dbg.print("(%s): triggering %ld (%d in batch)",
              m_description, m_queue->GetTopTimestamp(), c);
    m_queue->dequeueSnapshotEntry();
    c++;
  }
}

/*
 * Without taking any care about locking the mutex it is the same as InformInsert(GetTopTimestamp())
 */
void RACE_SnapshotTrigger::InformInsert()
{
  envdebug::component dbg(s_debug, "RACE_SnapshotTrigger::InformInsert");
  ACE_GUARD(ACE_Thread_Mutex, ace_mon, this->m_mutex);
  if(! m_queue->IsEmpty())
  {
    m_queue->DecrTopPending();
    dbg.print("%s: %d pending", m_description, m_queue->GetTopPending());
    InformInsertConclusion();
  }
  else
  {
    dbg.print("%s: m_queue is empty", m_description);
  }
}

void RACE_SnapshotTrigger::InformInsert(time_t timestamp)
{
  envdebug::component dbg(s_debug, "RACE_SnapshotTrigger::InformInsert(time_t)");
  ACE_GUARD(ACE_Thread_Mutex, ace_mon, this->m_mutex);
  RACE_SnapshotQueue::Entry *res = m_queue->GetEntry(timestamp);
  if(res!=NULL)
  {
    res->DecrPending();
    dbg.print("(%s,%ld) %d pending", m_description,timestamp,res->GetPending());
    InformInsertConclusion();
  }
  else
  {
    dbg.print("(%s,%ld) m_queue is empty", m_description, timestamp);
  }
}
