#ifndef RACE_THREAD_H
#define RACE_THREAD_H

#include <ace/Synch.h>

class envdebug;

//##Documentation
//## Simple wrapper for thread class with reference counting.
//## The thread reference count starts with one.
//## The child has to overwrite the run method.
class RACE_Thread
{
public:
  //##Documentation
  //## Constructor sets the name
  RACE_Thread(const char *threadname=0);

  //##Documentation
  //## gets the Name for debugging
  const char *name();
  
  //##Documentation
  //## starts the internal thread
  void start();

  //##Documentation
  //## release this thread object (Reference counting)
  void release();

  //##Documentation
  //## returns the number of refrences
  int number_of_refs();
  RACE_Thread *duplicate();
  static int wait_for_all_threads (const ACE_Time_Value *timeout = 0);
  //##Documentation
  //## returns if the internal thread is running
  bool is_running();
  //##Documentation
  //## waits for state changing (running/termination)
  //## implemented with condition-Variable
  void waitforStatechanging();
  //##Documentation
  //## joins the Thread
  void wait();
protected:
  static envdebug *debug;
  virtual void run() = 0;
  virtual ~RACE_Thread();
  //##Documentation
  //## Mutex for internal variables
  ACE_Thread_Mutex mutex_;
  //##Documentation
  //## signal when state changing
  ACE_Condition<ACE_Thread_Mutex> state_changed_;
private:
  //##Documentation
  //## Applicationname
  const char *name_;
  int thread_group_id;
  //##Documentation
  //## Thread id of running thread
  ACE_hthread_t tid_;

  //##Documentation
  //## native procedure which is executed by an thread
  //## It calls the run()-Method of this object
  static void *proc_to_run(void *myself);
  ACE_Thread_Mutex refcount_mutex_;
  int refcount_;
  bool isrunning_;
};
#endif
