#include"RACE_SnapshotTrigger.h"
#include<errorhandling/errorhandling.h>

class CMySnapshotCreator;

class CTestSnapshotEntry : public RACE_SnapshotQueue::Entry
{
public:
  CTestSnapshotEntry(time_t timestamp,int pending,CMySnapshotCreator *sc)
   : RACE_SnapshotQueue::Entry(timestamp,pending),mp_sc(sc) {}
  virtual void ConnectionLog();
private:
  CMySnapshotCreator *mp_sc;
};

class CMySnapshotCreator : public RACE_SnapshotCreator
{
public:
  CMySnapshotCreator(bool totalknown=false)
    :m_pendingtotal(0),m_pendingcurrent(0),m_totalknown(totalknown) {};
  virtual int GetPendingTotal();
  virtual int GetPendingForYou();
  virtual RACE_SnapshotQueue::Entry *GenNewEntry(time_t timestamp,int pending);
  void SetPending(int p) { m_pendingtotal=p; m_pendingcurrent=0; }
  void IncrPending(int a) { m_pendingcurrent=a;m_pendingtotal+=a; }
  void SetAndIncrPending(int b,int a)
  {
    m_pendingcurrent=a;
    m_pendingtotal=b+a;
  }
private:
  int m_pendingtotal;
  int m_pendingcurrent;
  int m_totalknown;
};


int CMySnapshotCreator::GetPendingTotal()
{
  return (m_totalknown ? m_pendingtotal : ST_UNKOWN);
}

int CMySnapshotCreator::GetPendingForYou()
{
  return (m_totalknown ? ST_UNKOWN : m_pendingcurrent);
}

void CTestSnapshotEntry::ConnectionLog()
{
  ERRLOG << "CTestSnapshotEntry::ConnectionLog() " << GetTimestamp() << std::endl;
  mp_sc->SetPending(0);
}

RACE_SnapshotQueue::Entry *CMySnapshotCreator::GenNewEntry(time_t timestamp,int pending)
{
  CTestSnapshotEntry *ret = new CTestSnapshotEntry(timestamp,pending,this);
  return ret;
}

int main(int argc,char **argv)
{
  RACE_LOGMSGBACKEND::instance()->init(argc,argv);
  bool totalknownmode = (argc>1 && strcmp(argv[1],"-t")==0);
  CMySnapshotCreator mysc (totalknownmode);
  RACE_SnapshotTrigger st (totalknownmode ? "test-totalknown" : "test-currentknown");

  ERRLOG << "Normal Operation" << std::endl;
  mysc.SetAndIncrPending(0,3);
  st.PrepareSnapshot(&mysc, 1);
  st.InformInsert();
  st.InformInsert();
  st.InformInsert();

  ERRLOG << (st.IsSnapshotPending() ? "ERROR, a snapshot is pending "
                                       : "OK, no snapshot is pending "
               )
            << std::endl;
  ERRLOG << "3 Snapshots " << std::endl;
  mysc.SetAndIncrPending(0,2);
  st.PrepareSnapshot(&mysc, 2);
  ERRLOG << (st.IsSnapshotPending() ? "OK, a snapshot is pending "
                         : "ERROR, no snapshot is pending "   )
            << std::endl;
  mysc.IncrPending(2);
  st.PrepareSnapshot(&mysc, 3);
  mysc.IncrPending(2);
  st.PrepareSnapshot(&mysc, 4);
  for(int i=0;i<6;i++)
  {
    st.InformInsert();
  }
  ERRLOG << (st.IsSnapshotPending() ? "ERROR, a snapshot is pending "
                                       : "OK, no snapshot is pending "
                )
            << std::endl;

  ERRLOG << "Mixed " << std::endl;
  mysc.SetAndIncrPending(0,3);
  st.PrepareSnapshot(&mysc, 5);
  st.InformInsert();
  st.InformInsert();
  mysc.SetAndIncrPending(1,2);
  st.PrepareSnapshot(&mysc, 6);
  st.InformInsert();
  st.InformInsert();
  st.InformInsert();
  ERRLOG << (st.IsSnapshotPending() ? "ERROR, a snapshot is pending "
                                       : "OK, no snapshot is pending "
                )
            << std::endl;

  ERRLOG << "Nothing pending" << std::endl;
   mysc.SetAndIncrPending(0,0);
  st.PrepareSnapshot(&mysc, 7);

  ERRLOG << "3 Snapshots with the same content " << std::endl;
  mysc.SetAndIncrPending(0,3);
  st.PrepareSnapshot(&mysc, 8);
  mysc.IncrPending(0);
  st.PrepareSnapshot(&mysc, 9);
  st.PrepareSnapshot(&mysc, 10);
  st.InformInsert();
  st.InformInsert();
  st.InformInsert();
  ERRLOG << (st.IsSnapshotPending() ? "ERROR, a snapshot is pending "
                                       : "OK, no snapshot is pending "
                )
            << std::endl;

  ERRLOG << "3 Snapshots with the same content, multiple Prepare " << std::endl;
  mysc.SetAndIncrPending(0,3);
  st.PrepareSnapshot(&mysc, 11);
  mysc.IncrPending(0);
  st.PrepareSnapshot(&mysc, 12);
  st.PrepareSnapshot(&mysc, 13);
  st.PrepareSnapshot(&mysc, 13);
  st.PrepareSnapshot(&mysc, 13);
  st.InformInsert();
  st.InformInsert();
  st.InformInsert();
  ERRLOG << (st.IsSnapshotPending() ? "ERROR, a snapshot is pending "
                         : "OK, no snapshot is pending "
  )
     << std::endl;

}
