#ifndef RACE_DEPOSITORY_H
#define RACE_DEPOSITORY_H
#include"RACE_LogfileBase.h"
#include"RACE_DepositoryNames.h"

//##Documentation
//## A Depository is an another kind of an RACE_Interface
//## Instead of growing indefinitly like an logfile,
//## a depository is compressed, the old instances are thown away.
class RACE_Depository : public RACE_LogfileBase, public RACE_DepositoryNames
{
public:
  RACE_Depository();
  virtual const char *GetBasename() const;
    //##Documentation
    //## Opens the Depository for entity_name
    //## returns true if recovery has to be done
  virtual void Open(const char *entity_name) throw (Systemerror);
  virtual void Open() throw (Systemerror);
  virtual void SwitchPart() throw (Systemerror);
  //##Documentation
  //## returns true if recovery has to be done
  bool WasRecovered() { return wasrecovered_; }
  virtual void Close();
  //##Documentation
  //## RACE_WriterInterface-Interface from Accumulator
  virtual int WriteV(const  struct  iovec  *  vector,  int count) throw (Systemerror);
  //##Documentation
  //## RACE_WriterInterface-Interface from Accumulator
  //## We have always only one part
  virtual int PartNo();

  //##Documentation
  //## Saves the Depository.dp in Depository.old.dp and the over files
  //## in the same way
  void MoveAway();

  //##Documentation
  //## Renames the former new file to the second File
  //## The Logfilereader reads it after the old
  //## Returns true it the Second file exists. It looks bad
  bool SaveUncompleteNew();

  //##Documentation
  //## Checks if a new empty Depository exists.
  //## If so, remove it
  bool  CheckAndRemoveZeroLengthNew();
  //##Documentation
  //## deletes the saved Depository
  void RemoveOld();

  //##Documentation
  //## deletes the Depository
  void Remove();

  //##Documentation
  //## reopens the File for Logfileswitching
  void Reopen();

private:
  int m_fd;
  bool wasrecovered_;
};
#endif
