#ifndef KVDEPOT_H
#define KVDEPOT_H
#include<RACE_Depository.h>
#include"RACE_ReaderDriver.h"

//##Documentation
//## Interface for Collectores from the view of a Depot
class CorrespondingCollector
{
public:
  //##Documentation
  //## All the content of the Collector is rewritten
  //## with the Logger in the new part
  virtual void RewriteAllAndClear()=0;
};

//##Documentation
//## Higher Layer - handles recovery
//##  Is a or has a Depository
//##  It is independed from content
class RACE_Depot : public RACE_Depository
{
public:
  RACE_Depot(CorrespondingCollector *correspondingcollector,
             RACE::Interpreter *interp,RACE_MessageVisitor *semantic)
    :collector_(correspondingcollector),oldlogdeletionpending_(false)
    {
      readerdriver_.SetInterpreter(interp);
      readerdriver_.SetIterationSemantic(semantic);
    }
  RACE::LogFileReader *Reader() { return readerdriver_.Reader(); }
  void SetSnapshotSemantik(RACE_MessageVisitor *semantic)
    {
      readerdriver_.SetSnapshotSemantic(semantic);
    }
  //##Documentation
  //## Open the Depository
  //## Note: It wold make sense to call Recover here, but
  //## the Depository is oppend before the logger, so that we havn't the
  //## Logger active at this time.
  virtual void Open(const char *name) throw (Systemerror);
  //##Documentation
  //## this is the reason of RACE_Depot is a RACE_Depository
  //## since the Logger should call this
  virtual void SwitchPart() throw (Systemerror);
  //##Documentation
  //## Here the Depot gets informed whan a snapshot is written.
  //## It deletes the old file if exists
  void InformSnapshot();

  //##Documentation
  //## reading the old Depository and feeding the collector
  void Recover();
private:
  CorrespondingCollector *collector_;
  RACE_ReaderDriver readerdriver_;
  bool oldlogdeletionpending_;
};
#endif
