#include"RACE_Message_Assembler.h"
#include<errorhandling/errorhandling.h>
#include<errorhandling/envdebug.h>

static envdebug debug("RACE_Message_Assembler");

RACE::Message_Assembler::Message_Assembler()
    :statemashine_(0)
{
}

RACE::Message_Assembler::Message_Assembler(Message_Observer *processor)
{
    statemashine_=new RACE::Message_Assembler::StateMashine(processor);
    RACE_MEMORY_FAIL(statemashine_==0,__FILE__,__LINE__,__FUNCTION__);
    statemashine_->init();
}

void RACE::Message_Assembler::input(char *buffer,int len)
{
    for(char *p=buffer;p<buffer+len;p++)
    {
	statemashine_->consume(*p);
    }
}

/*-----------State Mashine -------------------------*/
RACE::Message_Assembler::StateMashine::StateMashine()
    :processor_(0),currentblock_(0)
{
    collectmsg_state_=0;
    sendmsg_state_=0;
}

RACE::Message_Assembler::StateMashine::StateMashine(RACE::Message_Observer *processor)
    :processor_(processor),currentblock_(0),counter_(0)
{
    collectmsg_state_=0;
    sendmsg_state_=0;
}
void RACE::Message_Assembler::StateMashine::consume(char c)
{
    debug.print("StateMachine::Consume('%c'=%d)",c,c);
    counter_++;
    currentstate_->consume(c);
}

RACE_Message_Block *RACE::Message_Assembler::StateMashine::currentblock()
{
    return currentblock_;
}

void RACE::Message_Assembler::StateMashine::init()
{
    changeState(getBeginReadingState(),'\0');
}

RACE::Message_Assembler::State *
RACE::Message_Assembler::StateMashine::getCurrentState()
{
    return currentstate_;
}

void RACE::Message_Assembler::StateMashine::changeState(State *ns,char c)
{
    currentstate_=ns;
    ns->enter(c);
}

void RACE::Message_Assembler::StateMashine::allocateMessageBlock()
{
    if(currentblock_==0)
    {
	currentblock_ = new RACE_Message_Block(1024);
	RACE_MEMORY_FAIL(currentblock_==0,__FILE__,__LINE__,__FUNCTION__);
    }
}

void RACE::Message_Assembler::StateMashine::sendMessageBlock()
{
    processor_->process(currentblock_);
    currentblock_=0;
}

RACE::Message_Assembler::State *RACE::Message_Assembler::StateMashine::getCollectMessageState()
{
    if(collectmsg_state_==0)
    {
	collectmsg_state_=new CollectMessageState(this);
	RACE_MEMORY_FAIL(collectmsg_state_==0,__FILE__,__LINE__,__FUNCTION__);
    }
    return collectmsg_state_;
}

RACE::Message_Assembler::State *RACE::Message_Assembler::StateMashine::getSendMessageState()
{
    if(sendmsg_state_==0)
    {
	sendmsg_state_=new SendMessageState(this);
	RACE_MEMORY_FAIL(sendmsg_state_==0,__FILE__,__LINE__,__FUNCTION__);
    }
    return sendmsg_state_;
}

RACE::Message_Assembler::State *RACE::Message_Assembler::StateMashine::getBeginReadingState()
{
    return getCollectMessageState();
}

/*---------- State -----------*/
RACE::Message_Assembler::State::State(StateMashine *sm,const char *name)
    :sm_(sm),name_(name)
{
    
}

void RACE::Message_Assembler::State::enter(char)
{
    /* nothing */
}

/* CollectMessageState */
RACE::Message_Assembler::CollectMessageState::CollectMessageState(StateMashine *sm)
    :State(sm,"CollectMessage")
{
    
}

void RACE::Message_Assembler::CollectMessageState::enter(char)
{
    debug.print("Enter State CollectMessageState");
    sm_->allocateMessageBlock();
}

void RACE::Message_Assembler::CollectMessageState::consume(char c)
{
    debug.print("CollectMessageState: <- %c",c);
    *(sm_->currentblock()->wr_ptr(1))=c;
    if(c=='\n'||c=='\r')
    {
	sm_->changeState(sm_->getSendMessageState(),c);
    }
}

/* SendMessageState */
RACE::Message_Assembler::SendMessageState::SendMessageState(StateMashine *sm)
    :State(sm,"CollectMessage")
{
    
}

void RACE::Message_Assembler::SendMessageState::enter(char c)
{
    debug.print("Enter State SendMessageState");
    sm_->sendMessageBlock();
    // lamda - transition
    sm_->changeState(sm_->getBeginReadingState(),c);
}

void RACE::Message_Assembler::SendMessageState::consume(char c)
{
    debug.print("SendMessageState: <- %c",c);
    RACE_ERROR("SendMessageState: Transition not defined ");
}
