#ifndef RACE_MESSAGE_ASSEMBLER_H
#define RACE_MESSAGE_ASSEMBLER_H
#include<RACE_Message_Block.h>

namespace RACE
{
    //##Documentation
    //##  observer
    class Message_Observer
    {
    public:
	virtual void process(RACE_Message_Block *mb)=0;
    };

    //##Documentation
    //## assembles RACE_Message_Blocks from an input-device
    //## for protocolls, where the messagees are separated with
    //## a delimiter. 
    //## it uses the push modell,calls an Observer
    class Message_Assembler
    {
    public:
	class State;
	class StateMashine
	{
	public:
	    StateMashine(Message_Observer *processor);
	    //##Documentation
	    //## process a character in the finite state machine
	    void consume(char c);
		
	    // accessors
	    RACE_Message_Block *currentblock();

	    // action-Methods
	    //##Documentation
	    //## this method must be called,
	    //## but can be overwritten in ChildClasses
	    virtual void init();
	    
	    //##Documentation
	    //## chance the State, the state was entered with c
	    void changeState(State *newState,char c);
	    virtual void allocateMessageBlock();
	    void sendMessageBlock();

	    // states 
	    virtual State *getCollectMessageState();
	    virtual State *getSendMessageState();
	    // abstract state
	    //##Documentation
	    //## gets first State
	    //## Abstract State for trying to reuse
	    virtual State *getBeginReadingState();

	    State *getCurrentState();

	    size_t getcounter() { return (size_t)counter_; }
	protected:
	    StateMashine();
	    Message_Observer *processor_;
	    RACE_Message_Block *currentblock_;
	    State *currentstate_;
	    State *collectmsg_state_;
	    State *sendmsg_state_;
	    uint32_t counter_;
	};

	class State
	{
	public:
	    State(StateMashine *sm,const char *name);
	    virtual void consume(char c)=0;
	    //##Documentation
	    //## called when entered with input c
	    virtual void enter(char c);
	    
	protected:
	    StateMashine *sm_;
	private:
	    const char *name_;
	};

	class CollectMessageState : public State
	{
	public:
	    CollectMessageState(StateMashine *sm);
	    virtual void consume(char c);
	    virtual void enter(char c);
	};

	class SendMessageState : public State
	{
	public:
	    SendMessageState(StateMashine *sm);
	    virtual void consume(char c);
	    virtual void enter(char c);
	};
    public:
	Message_Assembler(Message_Observer *processor);
	void input(char *buffer,int len);
    protected:
	//##Documentation
	//## for derived MessageAssemblers only
	Message_Assembler();
	StateMashine *statemashine_;
    private:


   

    };
}
#endif
