#include"GrowingBuffer.h"
#include<string.h>
#include<stdio.h>
using namespace RAMS;

BufferOverflow::BufferOverflow(size_t free,size_t wanted):free_(free),wanted_(wanted)
  {
    snprintf(errmsg_,128,"BufferOverflow: %d wanted %d free",wanted_,free_);
  }

GrowingBuffer::GrowingBuffer(char *base,size_t size,size_t initial_size):Mseg(base,size)
{
    cur_len_ = &def_len_;
    def_len_ = initial_size;
    wr_ptr_=0;
}

GrowingBuffer::GrowingBuffer():Mseg()
{
    cur_len_ = &def_len_;
    def_len_ = 0;
    wr_ptr_  = 0;
}

void GrowingBuffer::init(char *base,size_t size,size_t initial_size)
{
    setto(base,size);
    cur_len_ = &def_len_;
    def_len_ = initial_size;
    wr_ptr_=0;
}


void GrowingBuffer::set_curlenptr(size_t *cl)
{
    size_t cursize=*cur_len_; // Remember the initial size
    cur_len_=cl;
    *cur_len_=cursize;
}

size_t GrowingBuffer::length()
{
    return *cur_len_;
}

size_t GrowingBuffer::freespace()
{
    return size() - *cur_len_;
}

void
GrowingBuffer::grow(size_t n) throw (BufferOverflow)
{
    if(n>freespace()) throw BufferOverflow(freespace(),n);
    *cur_len_ += n;
}

octet_t*
GrowingBuffer::wr_ptr ()
{
  return (octet_t *)(begin())+wr_ptr_;
}

octet_t*
GrowingBuffer::wr_ptr (octet_t *ptr) throw (BufferOverflow)
{
  size_t neu = ptr-(octet_t *)(begin());
  if (neu>length()) grow(neu - length());
  wr_ptr_ = neu;
  return wr_ptr ();
}

octet_t*
GrowingBuffer::wr_ptr (size_t n) throw (BufferOverflow)
{
  octet_t *ret=wr_ptr();
  wr_ptr_ += n;
  if (wr_ptr_>length()) grow(wr_ptr_ - length());
  return ret;
}

octet_t *GrowingBuffer::wrptr_align(short potenz2)
{
  octet_t *old=wr_ptr();
  long a=1<<potenz2;
  long mask=a-1;
  try
  {
     //RACE_COND_FAIL(sizeof(octet_t *)!=sizeof(long),"%s:%d:sizeof(octet_t *)!=sizeof(long)",__FILE__,__LINE__);
    wr_ptr((octet_t *)((long(old)+mask)&(~mask)));
    memset(old,'\0',wr_ptr()-old);
  }
  catch(BufferOverflow& ex)
  {
    wr_ptr(old);
  }
  return wr_ptr();
}

octet_t *GrowingBuffer::wr_ptr(off_t &retoff,short potenz2,size_t n) throw (BufferOverflow)
{
    octet_t *old=wr_ptr();
    long a=1<<potenz2;
    long mask=a-1; 
    wr_ptr((octet_t *)((long(old)+mask)&(~mask)));
    memset(old,'\0',wr_ptr()-old);
    retoff=wr_rel();
    return wr_ptr(n);
}

int
GrowingBuffer::copy (const octet_t *buf, size_t n) throw (BufferOverflow)
{
  memcpy(wr_ptr(n),buf,n);
  return n;
}

octet_t *
GrowingBuffer::ptr(size_t offset) throw (Readerror)
{
   if(offset >  *cur_len_) throw Readerror();
   return ((octet_t *)begin()+offset);
}
