#ifndef GROWINGBUFFER_H
#define GROWINGBUFFER_H
#include<exception>
#include<RAMS/mseg.h>
#include<RAMS/elementartypes.h>
namespace RAMS
{
//##Documentation
//## This exception is raised, when a block is full
//## One possible handling is to allocate a second block
class BufferOverflow : public std::exception
{
public:
  BufferOverflow(size_t free,size_t wanted);
  size_t free()   { return free_; }
  size_t wanted() { return wanted_; }
  virtual const char *what() const throw()
  {
    return errmsg_;
  }
protected:
  size_t free_;
  size_t wanted_;
  char errmsg_[128];
};

class Readerror {};

 /**
  * Enhances the segment with a current length 
  */
 class GrowingBuffer : public Mseg
 {
 public:
     GrowingBuffer();
     GrowingBuffer(char *base,size_t size,size_t initial_size=0);
     void init(char *base,size_t size,size_t initial_size=0);
     // to set the place, the length should recorded
     // Please only use in the construction phase
     void set_curlenptr(size_t *cl);
     size_t length();
     size_t freespace();
     void grow(size_t n) throw (BufferOverflow);

     //##Documentation
     //## relative position of the writepointer
     size_t wr_rel() { return wr_ptr_; }

     //##Documentation
     //##  Get the write pointer.
     octet_t *wr_ptr();
     
     //##Documentation
     //##  Set the write pointer to <ptr>.
     octet_t* wr_ptr(octet_t *ptr) throw (BufferOverflow);
     
     //##Documentation
     //##  Set the write pointer ahead <n> bytes.
     //##  and returns the old position.
     //##  It throws BufferOverflow if the fragment is full.
     //##  allows writing compactly things like memset(wr_ptr(10),10,c);
     octet_t* wr_ptr(size_t n) throw (BufferOverflow);
     
     //##Documentation
     //## aligns the wrptr to a boundary
     //## returns the new ptr
     octet_t *wrptr_align(short potenz2);

     //## Combination
     octet_t *wr_ptr(off_t &retoff,short potenz2,size_t n) throw (BufferOverflow);
     //##Documentation
     //##  Copies <n> bytes from <buf> into the Buffer starting at
     //##  the <wr_ptr> offset.  Return 0 and increment <wr_ptr> by <n> if
     //##  the method succeeds.  Returns -1 if the size of the message is
     //##  too small, i.e., for this to work correct, <end> must be >=
     //##  <wr_ptr>.
     int copy (const octet_t *buf, size_t n) throw (BufferOverflow);

     
     octet_t *ptr(size_t offset) throw (Readerror);
 protected:
     // Idea: the length should be written anywere in the segment ?
     // let this pointer point into the segment.
     size_t *cur_len_;
     // default location of above pointer
     size_t def_len_;
     size_t wr_ptr_; // Relative WritePointer
 };
};
#endif

