#ifndef PMSEG_H
#define PMSEG_H
/**
 * @file pmseg.h
 */
#include<signal.h>
#include<RAMS/mseg.h>

const unsigned long PAGESIZE=4096;
/**
 * @brief Protected memory segment
 *
 * Segment wird in Liste eingetragen. Bei einem Segmentation Fehler
 * wird diese durchsucht eine abstrakte Behandlung angestoen.
 * Somit knnen verschiedene Unterarten in einem Adressraum koexistiern.
 *
 * In dieser Behandlung kann beispielsweise eine geschieben-Markierung
 * gesetzt werden oder die Seite geloggt werden oder ...
 *
 * Ebenso k&ouml;nnte auf die verschiedenen segmentation-Faults verschieden
 * reagiert werden, z.B. bei fehlendem Objekt knnte ein Block gelesen werden
 */

class PMSeg: public RAMS::Mseg
{
  static PMSeg* wurzel;
  PMSeg *next;

  static void insert(PMSeg *diese);
  static void remove(PMSeg *diese);
public:
  static PMSeg* lookup(void *addr); // < 0 not found
public:
  PMSeg(): RAMS::Mseg() {}
  PMSeg(char *anfang,size_t lang);
  void set(char *anfang,size_t lang);
  virtual ~PMSeg();
  /// Callback f&uuml;r schutzfehler
  virtual void handle(RAMS::PageNo_t pageno,void *addr)=0;
protected: /// Hilfsfunktionen
  void protect(RAMS::PageNo_t pageno);
  void unprotect(RAMS::PageNo_t pageno);
  static void sigseg(int sig,siginfo_t *siginfo, void *unkown);
};
#endif
