#ifndef SHADOW_SEGMENT_H
#define SHADOW_SEGMENT_H
/**
 * @file segment.h
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A>
 *<A href="http://www.infnet.verein.de/">Informatik- und Netzwerkverein e.V</A>
 * Implementiert ein Schattenspeichersegment
 */
#include<errorhandling/errorhandling.h>
#include"pmseg.h"
#include"rpointer.h"
#include"bitmap.h"
#include"reliablesegment.h"

#define MAXPAGES 1024
/*--------- Speicherlayout ----------------------*/
typedef unsigned long Pageno;
typedef unsigned long index_t;
struct Pageentry
{
  unsigned long shadowbit:1;
  unsigned long pageno:31;
};

const unsigned long UNDEFINED_PAGE=0x7fffffff;

// PAGESIZE 4096 = 2^12
inline off_t index2offset(index_t in) { return in<<12; }
inline index_t offset2index(off_t o)  { return o>>12;  }

/**
 * @brief Verwaltet eine Version des Segments
 *
 * Verwaltung der Zuornung von logischer Seitennummer auf physikalische
 * Blocknummer
 */
class SegmentVersion
{
  size_t count_;
  Pageentry entry[1];
public:
  size_t maxcount() { return (PAGESIZE-sizeof(size_t))/sizeof(Pageentry); }
  Pageentry &operator[](index_t i) { return entry[i]; }
  size_t rsize() { return sizeof(size_t)+count_*sizeof(Pageentry); }
  size_t size() { return PAGESIZE; }
  size_t count() { return count_; }
  void grow(int anz)
  {
    RACE_COND_FAIL(count_+anz> maxcount(),"SegmentVersio::grow: to big");
    count_+=anz;
  }
  int clearallshadowbits();
  static SegmentVersion *init(char *puffer,int len);

  /// Fehlersuchausgabe
  void print(std::ostream& str);
};

/**
 * @brief Layout des Masters im Reliable Segment
 */
struct MasterLayout
{
  off_t mc;     // Offset Aktuelle Freispeicherbitmap
  off_t m0;     // Offset erste Freispeicherbitmap
  off_t m1;     // Offset zweite Freispeicherbitmap
  long  status; // Status des Segmente / eventuell spaeter Array
  // erste Freispeicherbitmap
  // zweite Freispeicherbitmap
};

/**
 * @brief Hilfsstrukturen fuer Shadowsegment
 * Wenn mehrere Segmente ein eine Datei sollen, dann knnte
 * dieses eine Factory werden.
 */
class ShadowSegMaster
{
  TwinblockStrategy tws;
  ReliableSegment rseg;
  RAMS::RPointer<BitMap> mc,m0,m1;
  // mc bzw. dessen offset ist der mapswitch
 public:
  ShadowSegMaster();
  /**
   * returns 1 if new, 0 if old, -1 if error
   */
  int open(char *name);
  BitMap& CurrentM() { return *(mc.ptr()); }
  BitMap& M0() { return *(m0.ptr()); }
  BitMap& M1() { return *(m1.ptr()); }
  void save();
  int getstatus() { return ((MasterLayout *)rseg.begin())->status; }
  void setstatus(int s) { ((MasterLayout *)rseg.begin())->status=s; }
  void copyandswitchm();
  void druckebelegtliste();
};

/**
 * @brief Shadowsegment
 */
class ShadowSegment : public PMSeg
{
  char svpuffer[2*PAGESIZE];
  SegmentVersion *vw[2];
  ShadowSegMaster ssm;
  /// Dateidescriptor Daten
  int sfd;
  /// Dateidescriptor V
  int vfd;
  SegmentVersion &Vwork() { return *vw[0]; }
  SegmentVersion &Vold() { return *vw[1]; }
  void readsegversions() throw(Systemerror);
  /**
   * Speichert Segmentversion nr auf Segmentversion snr
   * @param nr Nummer des Segmentes
   * @param snr Speicherplatz des Segmentes
   */
  void savesegversion(int nr,int snr) throw(Systemerror);
  void save();
  /// oeffnen nach Sicherung
  void reopen();
  /// nach dem ffnen die alten Seiten freigeben
  void clearshadowbitsandfree();

  /// recover im Fall, da eine ge&ouml;fnetes Segment vorliegt.
  void recover();

 public:
  ShadowSegment() { sfd=-1; vfd=-1; }
  void init();
  void open(const char *name) throw(Systemerror);

  void close()      { save(); }
  void checkpoint() { save(); reopen(); }
  void rollback();

  /// fuer PMseg
  void handle(RAMS::PageNo_t pageno,void *addr);
};
#endif





