/**
 * @file growingmseg.h
 * @brief wachsendes Speichersegment 
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2000 
 * Modifikation:
 *         16.3.2002 Ableitbar
 */
#ifndef GROWINGMSEG_H
#define GROWINGMSEG_H
#include<unistd.h>
#include<sys/mman.h>
#include<RAMS/mseg.h>
#include<errorhandling/errorhandling.h>

namespace RAMS
{

/**
 * \brief Wachsendes Speichersegment
 *
 * Dieses Speichersegment kann mit der Operation grow wachsen.
 * Das Segment ist mit 0 vorinitialiisert (was durch mapping auf /dev/zero
 * erreicht wird)<p>
 * Ableitung dieser Klasse koennen auch eine Datei hinterlegen.
 */
class Growingmseg: public Mseg
{
 public:
    class Error : public Systemerror
	{
	public:
	    Error(int err,char *pcall,char *msg,...):Systemerror()
		{
		    va_list args;
		    va_start(args,msg);
		    init(err,pcall,msg,args);
		    va_end(args);
		}
	};
 protected:

   int fd; /// Dateidescriptor
   int gmflags; /// Memoryflags
   /**
    * wird in abgeleiteten Klassen ueberschrieben
    */
   virtual void grow_internal(int anzseiten) throw(Error);
 public:
   Growingmseg():Mseg() { fd=-1; gmflags=MAP_PRIVATE; }
   /**
    * @param anzseiten Zahl der Speicherseiten, um die das Segment wachsen soll
    */
   void grow(int anzseiten) throw(Error);
   
   /**
    * Destructor: gibt Speichersegment wieder frei
    */
   virtual ~Growingmseg(); 
};

};
#endif



