#include"bitgrabber.h"
#include"errorhandling/errorhandling.h"

inline u_int32_t getwoff(size_t off,RAMS::octet_t b)
{
    return ((u_int32_t)b &(~(~0ul<<(8-off))));
}

inline u_int32_t getbitsimbyte(size_t off,size_t len, RAMS::octet_t b)
{
    size_t lm = (len < 8-off ? len : 8-off);
    size_t shift=(8-off-lm);
    u_int32_t ret=(b& ((~(~0ul<<len))<<shift));
    return (ret>>shift);
}

/* High-Endian only !!! */
u_int32_t
RAMS::BitGrabber::getbits(RAMS::bitoff_t bitoff,size_t len)
{
    u_int32_t ret=0;
    off_t byteoff=bitoff/8;
    off_t maxoff=byteoff+(len/8)+(len%8 > 0 ? 1 : 0);
    if((size_t)maxoff > seg_.size())
    {
	throw Error("BitGrabber::getbits: maxoff=%lu>seg.size=%d",
		    maxoff,seg_.size());
    }
    RAMS::octet_t *fb=(RAMS::octet_t *)(seg_.begin())+byteoff;
    
    bitoff_t bitrest=bitoff%8;
    size_t readbits=(8-bitrest > len ? len : 8-bitrest);
    ret=getbitsimbyte(bitrest,readbits,*fb);
    while(readbits<len)
    {
	fb++;
	bitrest=len-readbits;
	if(bitrest>8)
	{
	    ret<<=8;
	    ret|=*fb;
	    readbits+=8;
	}
	else
	{
	    ret<<=bitrest;
	    ret|=getbitsimbyte(0,bitrest,*fb);
	    readbits+=bitrest;
	}
    }
    return ret;
}

inline void setbitsimbyte(size_t off,size_t len, RAMS::octet_t &b,RAMS::octet_t set)
{
    size_t lm = (len < 8-off ? len : 8-off);
    RAMS::octet_t toset = (~(~0<<lm) & set);
    size_t shift=(8-off-lm);
    RAMS::octet_t mask  = ~(~0<<lm) << shift;
    b&=~mask; // clear bits
    b|=(toset<<shift);
}

inline u_int32_t masklast(size_t count)
{
    return ~(~0ul<<count);
}

void RAMS::BitGrabber::setbits(RAMS::bitoff_t bitoff,size_t len,u_int32_t value)
{
    off_t byteoff=bitoff/8;
    off_t maxoff=byteoff+(len/8)+(len%8 > 0 ? 1 : 0);
    if((size_t)maxoff > seg_.size())
    {
	throw Error("BitGrabber::setbits: maxoff=%lu>seg.size=%d",
		    maxoff,seg_.size());
    }
    RAMS::octet_t *fb=(RAMS::octet_t *)(seg_.begin())+byteoff;
    bitoff_t bitrest=bitoff%8;
    size_t setbits=(8-bitrest > len ? len : 8-bitrest); // min
    u_int32_t bruch = value;
    setbitsimbyte(bitrest,setbits,*fb,(RAMS::octet_t)(bruch & masklast(setbits)));
    bruch>>=setbits;
    while(setbits<len)
    {
	fb++;
	bitrest=len-setbits;
	if(bitrest>=8)
	{
	    RAMS::octet_t set=(RAMS::octet_t)(bruch & 255);
	    *fb=set;
	    bruch>>=8;
	    setbits+=8;
	}
	else
	{
	    RAMS::octet_t set=(RAMS::octet_t)(bruch & masklast(bitrest));
	    bruch>>=bitrest;
	    setbitsimbyte(0,bitrest,*fb,set);
	    setbits+=bitrest;
	}
    }
}
