#include"RACE_LogMsg_BackendMgr.h"
#include<errorhandling/errorhandling.h>
#include <sys/types.h>
#include <signal.h>

RACE_LogMsg_BackendMgr::RACE_LogMsg_BackendMgr()
    :wurzel_(0),
     progname_("<unknown>"),
     mainpid_(getpid())
{
    // setting default backend
    addBackend(new RACE_LogMsg_Bstderr());
}

RACE_LogMsg_BackendMgr::~RACE_LogMsg_BackendMgr()
{
   RACE_LogMsg_Backend *cur=wurzel_;
   while(cur!=0)
   {
       RACE_LogMsg_Backend *h=cur;
       cur=cur->next_;
       delete h;
   }
}

void 
RACE_LogMsg_BackendMgr::addBackend(RACE_LogMsg_Backend *backend)
{
    RACE_GUARD(ACE_Thread_Mutex,guard,mutex_,__FILE__,__LINE__,__FUNCTION__);
    backend->next_=wurzel_;
    wurzel_=backend;
}

void 
RACE_LogMsg_BackendMgr::clear()
{
    RACE_GUARD(ACE_Thread_Mutex,guard,mutex_,__FILE__,__LINE__,__FUNCTION__);
    RACE_LogMsg_Backend *cur=wurzel_;
    RACE_LogMsg_Backend *todel;
    wurzel_=0;
    while(cur!=0)
    {
	todel=cur;
	cur=cur->next_;
	delete todel;
    }
}

void 
RACE_LogMsg_BackendMgr::write(int priority,const char *buffer,int len)
{
    if(priority<=logupto_)
    {
	RACE_GUARD(ACE_Thread_Mutex,guard,mutex_,__FILE__,__LINE__,__FUNCTION__);
	for(RACE_LogMsg_Backend *b=wurzel_;b!=0;b=b->next_)
	{
	    b->write(priority,buffer,len);
	}
    }
}

void
RACE_LogMsg_BackendMgr::init(int /*argc*/,char **argv)
{
    progname_=argv[0];
    // facility
}

int RACE_LogMsg_BackendMgr::getMainPid() 
{
    return mainpid_;
}

const char *RACE_LogMsg_BackendMgr::progname()
{
    return progname_;
}

void
RACE_LogMsg_BackendMgr::abort()
{
    if(::kill(getMainPid(),SIGABRT)<0)
    {
	write(2,"KILL Failed\n",12);
    }
}
