#ifndef RACE_LOGMSG_BACKENDMGR_H
#define RACE_LOGMSG_BACKENDMGR_H
#include"RACE_LogMsg_Backend.h"
#include<ace/Synch.h>
#include<ace/Singleton.h>
//##Documentation
//## Distributes the log messages to diversal backends
class RACE_LogMsg_BackendMgr : public RACE_LogMsg_Backend
{
public:
    //##Documentation
    //## Adds a Backend
    //## The backend can have parameters
    void addBackend(RACE_LogMsg_Backend *backend);
    
    //##Documentation
    //## Implementation of a inherited abstract method
    void write(int priority,const char *buffer,int len);

    //##Documentation
    //## Information for the Thread specific loggers
    int getMainPid();

    //##Documentation
    //## send SIGABRT to main thread
    //## the first thread could be a signalhandling thread and
    //## catch the SIGABRT-Signal and termitate gently the process
    void abort();

    //##Documentation
    //## name of the calling program
    const char *progname();

    //##Documentation
    //## init has to be called very early in main()
    void init(int argc,char **argv);

    //##Documentation
    //## clear all backends
    void clear();
private:
    RACE_LogMsg_BackendMgr();
    ~RACE_LogMsg_BackendMgr();
    friend class ACE_Singleton<RACE_LogMsg_BackendMgr, ACE_Recursive_Thread_Mutex>;
    ACE_Thread_Mutex mutex_;
    //##Documentation
    //## simple C-Style list of backends
    RACE_LogMsg_Backend *wurzel_;
    const char *progname_;
    int mainpid_;
    
};

//##Documentation
//## we have one BackendMgr for the whole process
typedef ACE_Singleton<RACE_LogMsg_BackendMgr, ACE_Recursive_Thread_Mutex> RACE_LOGMSGBACKEND;
#endif
