#include"RACE_LogMsg_Stream.h"
#include<string>
#include<sstream>


ostream&
RACE_LogMsg_Stream::info(char *filename,int line,int op_status,int errnum)
{
    file(filename);
    linenum_=line;
    status_=op_status;
    errnum_=errnum;
    m_currentPriority = m_defaultPriority;
    return *this;
}

void
RACE_LogMsg_Stream::log_write(const char *s,std::streamsize n)
{
    const char *fmtstr=s;
    //CAUTION: we look behind the value
    if(fmtstr[n]!='\0')
    {
	char *workstr = (char *)alloca(n+1);
	if(!workstr)
	{
	    backend_->write(m_currentPriority,s,n);
	    backend_->write(CRIT,"Stack overflow\n",16);
	    return;
	} 
	memcpy(workstr,s,n);
	workstr[n]='\0';
	fmtstr=workstr;
    }
    this->log(m_currentPriority,fmtstr);
}

RACE_LogMsg_Stream::RACE_LogMsg_Stream()
    : RACE_ERRLOG::logbuf_init<char>(this),
      std::basic_ostream<char>(RACE_ERRLOG::logbuf_init<char>::buf_ptr()),
      backend_(RACE_LOGMSGBACKEND::instance()),
      m_defaultPriority(RACE_LogMsg_Stream::LDEBUG),
      m_currentPriority(m_defaultPriority)
{
}


void RACE_LogMsg_Stream::file(const char *filename)
{
    strncpy(file_,filename,MAXPATHLEN);file_[MAXPATHLEN]='\0';
}

const char *RACE_LogMsg_Stream::file()
{
    return file_;
}



ssize_t
RACE_LogMsg_Stream::log(Priority log_priority,
			  const ACE_TCHAR *format_str, ...)
{
  // Start of variable args section.
  va_list argp;
  va_start(argp,format_str);

  int result = this->log(format_str,
			 log_priority,
			 argp);
  va_end (argp);
  return result;
}


ssize_t RACE_LogMsg_Stream::log(const ACE_TCHAR *format_str,
               Priority log_priority,
               va_list argp)
{
  // We pick out additional Formats 
  std::ostringstream strbuf;
  const char *lf=format_str;
  while(*lf!='\0')
  {
      if(strncmp(lf,"@PROG",5)==0)
      {
	  strbuf << backend_->progname();
	  lf+=5;
      }
      else if(strncmp(lf,"%m",2)==0)
      {
	  strbuf << strerror(errnum_);
	  lf+=2;
      }
      else if(strncmp(lf,"@ERRSTR",7)==0)
      {
	  strbuf << strerror(errnum_);
	  lf+=7;
      }
      else if(strncmp(lf,"@LINE",5)==0)
      {
	  strbuf << linenum_;
	  lf+=5;
      }
      else if(strncmp(lf,"@FILE",5)==0)
      {
	  strbuf << file();
	  lf+=5;
      }
      else if(strncmp(lf,"@PID",4)==0)
      {
	  strbuf << backend_->getMainPid();
	  lf+=4;
      }
      else if(strncmp(lf,"@TID",4)==0)
      {
          strbuf << ACE_Thread::self();
          lf+=4;	
      } 
      else if(strncmp(lf,"@TIME",5)==0)
      {
	  lf+=5;
      }
      else
      {
	  strbuf << *lf;
	  lf++;
      }
  }
  if(*(lf-1)!='\n')
  {
      strbuf << '\n';
  }
  std::string newformat=strbuf.str();
  const char *newformatstr=newformat.c_str();
  size_t result=vsnprintf(msg_,RACE_MAXLOGMSGLEN,newformatstr,argp);
  backend_->write(log_priority,msg_,result);
  return result;
}

#define TOLOG(PRIORITY) \
  va_list argp; \
  va_start(argp,format_str); \
  this->log(format_str,PRIORITY,argp); \
  va_end (argp);

#define COND_TOLOG(PRIORITY) \
if(validity) \
{ \
  va_list argp; \
  va_start(argp,format_str); \
  this->log(format_str,PRIORITY,argp); \
  va_end (argp); \
}

void
RACE_LogMsg_Stream::debug (char *format_str, ...)
{
    TOLOG(LDEBUG)
}

void 
RACE_LogMsg_Stream::message (char *format_str, ...)
{
    TOLOG(INFO)
}

void 
RACE_LogMsg_Stream::warn (char *format_str, ...)
{
    TOLOG(WARNING);
}

void 
RACE_LogMsg_Stream::error (char *format_str, ...)
{
    TOLOG(ERROR);
}


void
RACE_LogMsg_Stream::report (char *format_str, ...)
{
    TOLOG(NOTICE)
}
  
void 
RACE_LogMsg_Stream::fail (char *format_str, ...)
{
    TOLOG(EMERGENCY)
    backend_->abort();	
}
                                                                                
void
RACE_LogMsg_Stream::cond_debug (bool validity, char *format_str, ...)
{
  COND_TOLOG(LDEBUG)
}

void 
RACE_LogMsg_Stream::cond_message (bool validity, char *format_str, ...)
{
    COND_TOLOG(NOTICE)
}

void RACE_LogMsg_Stream::cond_warn (bool validity, char *format_str, ...)
{
    COND_TOLOG(WARNING)
}


void RACE_LogMsg_Stream::cond_error (bool validity, char *format_str, ...)
{
    COND_TOLOG(ERROR)
}

void RACE_LogMsg_Stream::cond_fail (bool validity, char *format_str, ...) 
{
    if(validity)
    {
	TOLOG(EMERGENCY)
	::abort();
	//backend_->abort();
    }
}


std::ostream&  operator<<(std::ostream & os, RACE_LogMsg_Stream::Priority priority)
{
    RACE_LogMsg_Stream * slog = dynamic_cast<RACE_LogMsg_Stream *>(&os);
    
    if (slog)
    {
	slog->m_currentPriority = priority;
    }

    return os;
 }
