#include <string>
#include <cstdlib>
#include <cstdio>
#include <cstdarg>

#include <errorhandling/envdebug.h>
#include <errorhandling/env_access.h>
#include <errorhandling/RACE_LogMsg_Stream.h>

envdebug::envdebug (const char *classname)
{
  // Example: for a classname "RtecSupplier",
  // an environment variable DBG_RtecSupplier
  // has to be defined to switch on debug messages.
  env_level dbg_all_flag ("DBG_ALL");
  if (dbg_all_flag.is_on ()) {
    _dbg = true;
    _level =  dbg_all_flag.level();
  }
  else {
    const char *prefix = "DBG_";
    char env_var[strlen(classname) + strlen(prefix) + 1];
    sprintf (env_var, "%s%s", prefix, classname);
    env_level dbg_special_flag (env_var);
    _dbg = dbg_special_flag.is_on ();
    _level = dbg_special_flag.level();
  }
  env_flag no_timestamps_flag ("DBG_NO_TIMESTAMPS");
  if (! no_timestamps_flag.is_on ())
  {
      RACELOGMSGSTREAM::instance()->time_output_on();
  }
}

void envdebug::print(char *format_str, ...)
{
  if (! _dbg)
    return;
  RACE_COND_FAIL (format_str == NULL, "envdebug::print got NULL msg");
  va_list argp;
  va_start(argp,format_str); 
  RACELOGMSGSTREAM::instance()->log(format_str,RACE_LogMsg_Stream::LDEBUG,argp); \
  va_end (argp);
}

void envdebug::vprint(char *format_str,va_list argp)
{
   RACELOGMSGSTREAM::instance()->log(format_str,RACE_LogMsg_Stream::LDEBUG,argp);
}

envdebug::component::component(envdebug* pe, const char *name)
  : m_pe(pe), m_name(name)
{
  RACE_COND_FAIL(m_pe == NULL, "envdebug::component::component: no envdebug pointer given");
  m_pe->print("%s: enter\n",name);
}

envdebug::component::~component()
{
  m_pe->print("%s: leave\n",m_name);  
}

void envdebug::component::print(char *msg, ...)
{
  if (! m_pe->is_on ())
    return;
  RACE_COND_FAIL(msg == NULL, "envdebug::print got NULL msg");
  char *format_str = (char *)alloca(strlen(msg)+strlen(m_name)+5);
  RACE_COND_FAIL(format_str==NULL,"%s:%d: alloca failed",__FILE__,__LINE__);
  strcpy(format_str,m_name);
  strcat(format_str,": ");
  strcat(format_str,msg);
  va_list vargu;
  va_start (vargu, msg);
  m_pe->vprint(format_str,vargu);
  va_end (vargu);
}


