/*************************************************************************
 * Defmodul...: errorclass.h                                             *
 * Zweck......: einfache Fehlerobjecte für try-throw-catch               *
 * Copyright..: 1998 Informatik- und Netzwerkverein Ravensburg e.V       *
 *              nach GNU-Copyleft                                        *
 * Author.....: Rudolf Weber                                             *
 * Bemerkung..: Stammt aus radiusd                                       *
 * 2004: Abgeleitet aus std::exception                                   *
 *************************************************************************/
#ifndef ERRORCLASS_H
#define ERRORCLASS_H
#include<stdarg.h>
#include<stdio.h>
#include<string.h>
#include<exception>

//##Documentation
//## This Exception is raised when no more memory can be allocated
class NoMoreDynamicMemory : public std::exception
{
public:
  NoMoreDynamicMemory(const char *file,long line,const char *function)
  {
    snprintf(errmsg_,sizeof(errmsg_),"No more Memory at %s:%ld in %s",file,line,function);
  }
  virtual const char *what() const throw() { return errmsg_; }
private:
  char errmsg_[128];
};

class MutexFail : public std::exception
{
public:
  MutexFail(const char *file,long line,const char *function)
  {
    snprintf(errmsg_,sizeof(errmsg_),"Mutex failed at %s:%ld in %s",file,line,function);
  }
  virtual const char *what() const throw() { return errmsg_; }
private:
  char errmsg_[128];
};

// Fehlerbehandlungsklassen
class Error : public std::exception
{
protected:
  char errmsg[128];
public:
  Error() {strcpy(errmsg,"Unknown");}
  Error(const char *msg,...) __attribute__ ((format(printf,2,3)));
  virtual const char *what() const throw() { return errmsg; }
};

/**
 * @brief Systemcallerror
 */
class Systemerror: public Error
{
    int errnr;
    const char *icall;
public:
    Systemerror() {}
    /**
     * Konstruktor mit variablel vielen Argumenten
     * unterst&uuml;zt zusätzlich noch %m f&uuml;r Systemfehlermeldung
     * @param err Fehlercode errno
     * @param pcall Name des systemcalls
     * @param msg   Fehlermeldungsformatstring
     */
    Systemerror(int err,char *pcall,char *msg,...)
        {
            va_list args;
            va_start(args,msg);
            init(err,pcall,msg,args);
            va_end(args);
        }


    /**
     * Initialisierung mit variablel vielen Argumenten
     * unterst&uuml;zt zusätzlich noch %m f&uuml;r Systemfehlermeldung
     * @param err Fehlercode errno
     * @param pcall Name des systemcalls
     * @param msg   Fehlermeldungsformatstring
     */
    void init(int err,char *pcall,char *msg,va_list args)
        {
            icall = pcall;
            const int format_sp_size = 128;
            char format_sp[format_sp_size];
            char *format=msg;
            char *pm=strstr(msg,"%m");
            register int l = pm-format;
            if(pm!=NULL && l<120)
            {
                char *pq,*pz;
                strncpy(format_sp,msg,l);
                pq = strerror(err);
                pz=format_sp+l;
                while(pz-format_sp < format_sp_size && *pq != '\0') *pz++=*pq++;
                pq=pm+2;
                while(pz-format_sp < format_sp_size && *pq != '\0') *pz++=*pq++;
                *pz='\0';
                format = format_sp;
            }
            vsnprintf(errmsg,format_sp_size,format,args);
        }
    /**
     * liefert Errno
     */
    int errornr() { return errnr; }
    /**
     * liefert Name des Systemcalls
     */
    const char *call() { return icall; }
};

class Syntaxerror: public Error
{
public:
  Syntaxerror(const char *m,...) __attribute__ ((format(printf,2,3)));
};

class Databaseerror: public Error
{
  long icode;
public:
  Databaseerror(const char *m):Error(m){};
  Databaseerror(char *dbname,long code,char *m,...) __attribute__ ((format(printf,4,5)));
  long code() const { return icode; }
};
#endif

