#include"KVSnapshotDumpable.h"
#include<errorhandling/envdebug.h>

envdebug *debug=new envdebug("KVSnapshotDumpable");

KVSnapshotDumpable::KVSnapshotDumpable():RACE_Dumpable(4096),
                                         snapshottime_(0),lsc_(0)
{
  Logfile::Header *hptr = (Logfile::Header *)base();
  wr_ptr(sizeof(Logfile::Header));
  hptr->len=length();
  hptr->elementtype = Logfile::Snapshot_Type;
  length_ptr = &(hptr->len);
}

KVSnapshotDumpable::KVSnapshotDumpable(RACE_Data_Block *db,bool frag,RACE_LogfileOffset lo)
  :RACE_Dumpable(db,frag,lo),snapshottime_(0),lsc_(0)
{
  Logfile::Header *hptr = (Logfile::Header *)base();
  length_ptr = &(hptr->len);
}

void KVSnapshotDumpable::CallVisitor(RACE_MessageVisitor *visitor)
{
  KVSemantik *sem=dynamic_cast<KVSemantik *>(visitor);
  if(sem)
  {
    sem->KeyValueSnapshot(this);
  }
}

Logfile::KeyRecord *KVSnapshotDumpable::append(const Gdat& key)
{
  size_t addlen = sizeof(Logfile::KeyRecord)+key.get_len();
  int retry=2;
  while(retry>0)
  {
    try
    {
      Logfile::KeyRecord *ret = (Logfile::KeyRecord *)wr_ptr(addlen);
      ret->reclen=wrptr_align(2)-(char *)ret;
      ret->keylen=key.get_len();
      memcpy(ret->key,key.get_const_data(),key.get_len());
      ret->offset.part=0;ret->offset.offset=0;
      *length_ptr+=ret->reclen;
      retry=0;
      return ret;
    }
    catch(BufferOverflow &bo)
    {
      debug->print("KVSnapshotDumpable::append: Bufferoverflow wanted=%d free=%d",                  bo.wanted(),bo.free());
      expand();
      retry--;
    }
  }
  RACE_FATAL_ERROR("KVSnapshotDumpable::append: Can't append key with length %d",key.get_len());
  return (Logfile::KeyRecord *)0;
}

Logfile::KeyRecord *KVSnapshotDumpable::first_key()
{
  if(*length_ptr == sizeof(Logfile::KVSnapshotEntry))
  {
    return NULL;
  }
  begin();
  rd_ptr(sizeof(Logfile::KVSnapshotEntry));
  return (Logfile::KeyRecord *)rd_ptr();
}

Logfile::KeyRecord *KVSnapshotDumpable::next_key()
{
  Logfile::KeyRecord *current_keyrec_ = (Logfile::KeyRecord *)rd_ptr();
  RACE_COND_FAIL(current_keyrec_==NULL,"KVSnapshotDumpable::next_key(): Not Initialized");
  RACE_COND_FAIL(current_keyrec_->reclen < sizeof(Logfile::KeyRecord),"KVSnapshotDumpable::next_key(): too short record");
  rd_ptr(current_keyrec_->reclen);
  if(remaining()==0)
  {
    advance();
    if(!is_done())
    {
      return (Logfile::KeyRecord *)rd_ptr();
    }
    else
    {
      return NULL;
    }
  }
  return (Logfile::KeyRecord *)rd_ptr();
}

void KVSnapshotDumpable::InformOffset(RACE_LogfileOffset offset)
{
  if(lsc_)
  {
    lsc_->SetOffset(offset,snapshottime_);
  }
  else
  {
    RACE_WARN("KVSnapshotDumpable::InformOffset: lsc_==NULL, ignoring ");
  }
}

void KVSnapshotDumpable::InformCompleted()
{
  if(lsc_)
  {
    lsc_->InformSnapshotWritten();
  }
  else
  {
    RACE_WARN("KVSnapshotDumpable::InformCompleted: lsc_==NULL, ignoring ");
  } 
}

void KVSnapshotDumpable::Discard()
{
  if(lsc_)
  {
    lsc_->InformDiscard(snapshottime_);
  }
  else
  {
    RACE_WARN("KVSnapshotDumpable::InformDiscard: lsc_==NULL, ignoring ");
  }
}

KVSnapshotDumpable::KVSnapshotDumpable(char):length_ptr(0),snapshottime_(0),lsc_(0)
{

}

RACE_Message_Block *KVSnapshotDumpable::CloneManagementData() const
{
  char x='X';
  KVSnapshotDumpable *ret=new KVSnapshotDumpable(x);
  RACE_COND_FAIL(ret==NULL,"RACE_Message_Block *CloneManagementData():No Memory");
  return ret;
}

