#include<iostream>
#include<stdlib.h>
#include<string.h>
#include<ctype.h>
#include"gdat.h"
#include<assert.h>

Gdat::Gdat()
{
  umem=NULL;
  len=alen=ulen=0;
  allocmem=NULL;
}

Gdat::Gdat(char *usermem,size_t usermemlen)
{
  umem=usermem;
  len=ulen=(usermem ? usermemlen : 0 );
  allocmem=NULL;alen=0;
}

Gdat::Gdat(const Gdat &other) throw(Error)
{
  umem=NULL;
  ulen=0;
  len=alen=other.get_len();
  allocmem=(char *)malloc(alen);
  if(allocmem==NULL) throw Error("%s:%d: No Memory",__FILE__,__LINE__);
  memcpy(allocmem,other.get_const_data(),alen);
}

Gdat& Gdat::operator=(const Gdat &other) throw(Error)
{
  if(this!=&other)
  {
    frei();
    umem=NULL;
    ulen=0;
    len=alen=other.get_len();
    allocmem=(char *)malloc(alen);
    if(allocmem==NULL) throw Error("%s:%d: No Memory",__FILE__,__LINE__);
    memcpy(allocmem,other.get_const_data(),alen);
  }
  return *this;
}

void Gdat::frei()
{
  if(allocmem)
  {
    free(allocmem);
    allocmem=NULL;alen=0;
  }
}

void Gdat::set(char *usermem,size_t lang)
{
  frei();
  umem=usermem;
  ulen=len=lang;
}

void Gdat::set(datum &d,PSpeichertyp st) throw(Error)
{
  frei();
  if(st==PST_STATIC)
  {
    char *zptr=umem;
    if(d.dsize > ulen || umem==NULL)
    {
      allocmem=(char *)malloc(d.dsize);
      alen=d.dsize;
      if(allocmem==NULL) throw Error("%s:%d: No Memory",__FILE__,__LINE__);
      zptr=allocmem;
    }
    len=d.dsize;
    memcpy(zptr,d.dptr,d.dsize);
  }
  else
  {
    allocmem=d.dptr;
    len=alen=d.dsize;
  }
}

#ifdef WITH_DB_CXX
void Gdat::set(Dbt &d)
{
  frei();
  if(d.get_flags()&DB_DBT_MALLOC)
  {
    allocmem=(char *)(d.get_data());
    alen=len=d.get_dlen();
  }
  else
  {
    assert(d.get_data()==umem);
  }
}
#endif
#ifdef WITH_DB
void Gdat::set(DBT &d)
{
  frei();
  if(d.flags&DB_DBT_MALLOC)
  {
    allocmem=(char *)(d.data);
    alen=len=d.size;
  }
  else
  {
    umem=(char *)(d.data);
    ulen=d.ulen;
    len=d.size;
  }
}

void Gdat::exportmem(DBT &d)
{
  frei();
  memset(&d,'\0',sizeof(DBT));

  if(umem!=NULL)
  {
    d.data=umem;
    d.size=len;
    d.ulen=ulen;
    d.flags=DB_DBT_USERMEM;
  }
  else
  {
    d.flags=DB_DBT_MALLOC;
  }
}

void Gdat::export_val(DBT &d)
{
  memset(&d,'\0',sizeof(DBT));
  if(allocmem!=NULL)
  {
    d.data=allocmem;
    d.size=len;
    d.ulen=alen;
    d.flags=DB_DBT_MALLOC;
  }
  else
  {
    d.data=umem;
    d.size=len;
    d.ulen=ulen;
    d.flags=DB_DBT_USERMEM;
  }
}
#endif
void Gdat::set_len(size_t lang) throw (Error)
{
  if(allocmem != NULL)
  {
    if(lang>alen)
    {
      throw Error("gdat::setlen(%d): zu gro %d>%d",lang,lang,alen);
    }
  }
  else
  {
    if(lang>ulen)
    {
      throw Error("gdat::setlen(%d): zu gro %d>%d",lang,lang,ulen);
    }
  }
  len=lang;
}

void *Gdat::garantie(size_t l) throw(Error)
{
  if(umem != NULL && ulen >= l) return umem;
  if(allocmem != NULL && alen >= l) return allocmem;
  frei();
  if((l&0xf)!=0) { l=((l>>4)+1)<<4; } // Nur 16er Blcke
  allocmem=(char *)malloc(l);
  if(allocmem==NULL) throw Error("%s:%d: No Memory",__FILE__,__LINE__);
  alen=l;
  return allocmem;
}

void Gdat::exchangetake(Gdat& d)
{
  frei();
  if(d.allocmem != NULL)
  {
    allocmem=d.allocmem;len=alen=d.alen;
    d.take();
  }
  else
  {
    char *h = umem;
    size_t hulen = ulen;
    size_t hlen = len;
    umem=d.umem;d.umem = h;
    ulen=d.ulen;d.ulen = hulen;
    len=d.len;d.len  = hlen;
  }
}

std::ostream& operator<<(std::ostream& ostr,Gdat& d)
{
  if(d.get_data()!=NULL)
  {
    for(char *p=(char *)d.get_data();p<(char *)d.get_data()+d.get_len();p++)
    {
      if(isprint(*p))
      {
        ostr << *p;
      }
      else
      {
        ostr << '\\' << std::oct << (int)*p << std::dec ;
      }
    }
  }
  else
  {
    ostr << "(null)";
  }
  return ostr;
}

