#include"RACE_Log_Msg_Stream.h"

ostream&
RACE_Log_Msg_Stream::info(char *file,int line,int op_status,int errnum)
{
    acelog_->conditional_set(file,line,op_status,errnum);
    return *this;
}

RACE_Log_Msg_Stream::RACE_Log_Msg_Stream()
    : std::ostream(m_strbuf.rdbuf()),
      acelog_(ACE_Log_Msg::instance()),
      m_defaultPriority(LM_DEBUG),
      m_currentPriority(m_defaultPriority)
{
}

int 
RACE_Log_Msg_Stream::open (const ACE_TCHAR *prog_name,u_long flags,
			   const ACE_TCHAR *logger_key)
{
    return acelog_->open(prog_name,flags,logger_key);
}

ssize_t
RACE_Log_Msg_Stream::log (ACE_Log_Priority log_priority,
			  const ACE_TCHAR *format_str, ...)
{
  // Start of variable args section.
  va_list argp;
  va_start(argp,format_str);

  int result = acelog_->log (format_str,
			     log_priority,
			     argp);
  va_end (argp);
  return result;
}

ssize_t RACE_Log_Msg_Stream::log (const ACE_TCHAR *format_str,
               ACE_Log_Priority log_priority,
               va_list argp)
{
  int result = acelog_->log (format_str,
			     log_priority,
			     argp);
  return result;
}

#define TOLOG(PRIORITY) \
  va_list argp; \
  va_start(argp,format_str); \
  acelog_->log(format_str,PRIORITY,argp); \
  va_end (argp);

#define COND_TOLOG(PRIORITY) \
if(validity) \
{ \
  va_list argp; \
  va_start(argp,format_str); \
  acelog_->log(format_str,PRIORITY,argp); \
  va_end (argp); \
}

void
RACE_Log_Msg_Stream::debug (char *format_str, ...)
{
    TOLOG(LM_DEBUG)
}

void 
RACE_Log_Msg_Stream::message (char *format_str, ...)
{
    TOLOG(LM_INFO)
}

void 
RACE_Log_Msg_Stream::warn (char *format_str, ...)
{
    TOLOG(LM_WARNING);
}

void 
RACE_Log_Msg_Stream::error (char *format_str, ...)
{
    TOLOG(LM_ERROR);
}

void
RACE_Log_Msg_Stream::report (char *format_str, ...)
{
    TOLOG(LM_NOTICE)
}
  
void 
RACE_Log_Msg_Stream::fail (char *format_str, ...)
{
    TOLOG(LM_EMERGENCY)
}
                                                                                
void
RACE_Log_Msg_Stream::cond_debug (bool validity, char *format_str, ...)
{
  COND_TOLOG(LM_DEBUG)
}

void 
RACE_Log_Msg_Stream::cond_message (bool validity, char *format_str, ...)
{
    COND_TOLOG(LM_NOTICE)
}

void RACE_Log_Msg_Stream::cond_warn (bool validity, char *format_str, ...)
{
    COND_TOLOG(LM_WARNING)
}

void RACE_Log_Msg_Stream::cond_error (bool validity, char *format_str, ...)
{
    COND_TOLOG(LM_ERROR)
}

void RACE_Log_Msg_Stream::cond_fail (bool validity, char *format_str, ...) 
{
    COND_TOLOG(LM_EMERGENCY)
}

std::ostream &
operator<<(std::ostream & os, RACE_Log_Msg_Stream::Control control)
{
    RACE_Log_Msg_Stream * rlms = dynamic_cast<RACE_Log_Msg_Stream *>(&os);
    os << std::endl;  
    if (rlms)
    {
	switch (control)
	{
	    case RACE_Log_Msg_Stream::eom:
		rlms->log(rlms->m_currentPriority,rlms->m_strbuf.str().c_str());
		rlms->m_strbuf.str("");
		rlms->m_currentPriority = rlms->m_defaultPriority;
		break;
		
	    case RACE_Log_Msg_Stream::clr:
		rlms->m_strbuf.str("");
		rlms->m_currentPriority = rlms->m_defaultPriority;
		break;
		
	    default:
		rlms->log(LM_ERROR,"RACE_Log_Msg_Stream::operator << (const Control control): "
			" Unknown control %i.", 
			control);
		break;
	}
    }
    return os;
}

std::ostream&  operator<<(std::ostream & os, ACE_Log_Priority priority)
{
    RACE_Log_Msg_Stream * slog = dynamic_cast<RACE_Log_Msg_Stream *>(&os);
    
    if (slog)
    {
	slog->m_currentPriority = priority;
    }

    return os;
 }
