/*
 * $Id: env_access_level_test.cpp,v 1.3 2004/04/22 15:18:36 okellogg Exp $
 */
#include "utility/env_access.h"
#include "utility/output.h"

class thc { // the "test helper class"
  public:
    thc (const char* envname,
         const char* content,
         unsigned long expected_level)
      : name (envname), value (content), level (expected_level) {}
    const char* name;
    const char* value;
    unsigned long level;

    void test () const;
};

void
thc::test () const
{
  // check usage
  output::cond_fail (name == NULL, "thc::test: missing name");

  // prepare environment
  if (value)
    output::cond_fail (setenv (name, value, 1) == -1, "thc::test: setenv failed");
  else
    unsetenv (name);

  // create env_levels
  env_level my_level_env (name);
  unsigned long my_level = my_level_env.level ();
  bool my_level_state = my_level_env.is_on ();

  // check my_level and my_level_state
  output::cond_fail (my_level != level,
                     "thc::test: %s: did not find expected level %u - found %u", name, level, my_level);
  output::cond_fail ((my_level && ! my_level_state) || ((my_level == 0) && my_level_state),
                     "thc::test: %s: mismatch in activation state - level is %u and activation is %s",
                     name, my_level, my_level_state ? "ON" : "OFF");

  output::message ("thc::test: %s: SUCCESS", name);
}

#define NUMBER_OF_TESTITEMS 10

static const thc thc_array[NUMBER_OF_TESTITEMS] =
{
  thc ("TESTVAR_0A", NULL, 0),
  thc ("TESTVAR_0B", "", 0),
  thc ("TESTVAR_0C", "0", 0),
  thc ("TESTVAR_0D", "0000", 0),
  thc ("TESTVAR_0E", "3333333333333333333333333333333333333333333333333", 0), // Too long!
  thc ("TESTVAR_0F", "abcd7", 0),
  thc ("TESTVAR_MI", "-2", (unsigned long)(-2)), // This gives a valid number!
  thc ("TESTVAR_1A", "1", 1),
  thc ("TESTVAR_1B", "1abcd", 1),
  thc ("TESTVAR_2A", "2", 2)
};

int
main ()
{
  for (unsigned i = 0; i < NUMBER_OF_TESTITEMS; i++)
    thc_array[i].test ();
  return 0;
}

